@@ -22,6 +22,10 @@
 #include <libdvbv5/descriptors.h>
 #include <libdvbv5/dvb-fe.h>
 
+#if __GNUC__ >= 9
+#pragma GCC diagnostic ignored "-Waddress-of-packed-member"
+#endif
+
 void dvb_table_header_init(struct dvb_table_header *t)
 {
 	bswap16(t->bitfield);
