@@ -18,14 +18,14 @@
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <sstream>
+#include <cstdlib>
 
 #include "cec-compliance.h"
+#include "compiler.h"
 #ifndef ANDROID
 #include "version.h"
 #endif
 
-#include "cec-table.h"
-
 /* Short option list
 
    Please keep in alphabetical order.
@@ -39,10 +39,14 @@ enum Option {
 	OptColor = 'C',
 	OptSetDevice = 'd',
 	OptSetDriver = 'D',
+	OptExpect = 'e',
 	OptExitOnFail = 'E',
+	OptTestFuzzing = 'F',
 	OptHelp = 'h',
 	OptInteractive = 'i',
-	OptNoWarnings = 'n',
+	OptListTests = 'l',
+	OptExpectWithNoWarnings = 'n',
+	OptNoWarnings = 'N',
 	OptRemote = 'r',
 	OptReplyThreshold = 'R',
 	OptSkipInfo = 's',
@@ -119,6 +123,9 @@ static struct option long_options[] = {
 	{"exit-on-fail", no_argument, 0, OptExitOnFail},
 	{"exit-on-warn", no_argument, 0, OptExitOnWarn},
 	{"remote", optional_argument, 0, OptRemote},
+	{"list-tests", no_argument, 0, OptListTests},
+	{"expect", required_argument, 0, OptExpect},
+	{"expect-with-no-warnings", required_argument, 0, OptExpectWithNoWarnings},
 	{"timeout", required_argument, 0, OptTimeout},
 	{"trace", no_argument, 0, OptTrace},
 	{"verbose", no_argument, 0, OptVerbose},
@@ -129,6 +136,7 @@ static struct option long_options[] = {
 	{"reply-threshold", required_argument, 0, OptReplyThreshold},
 
 	{"test-adapter", no_argument, 0, OptTestAdapter},
+	{"test-fuzzing", no_argument, 0, OptTestFuzzing},
 	{"test-core", no_argument, 0, OptTestCore},
 	{"test-audio-rate-control", no_argument, 0, OptTestAudioRateControl},
 	{"test-audio-return-channel-control", no_argument, 0, OptTestARCControl},
@@ -172,7 +180,7 @@ static struct option long_options[] = {
 	{0, 0, 0, 0}
 };
 
-static void usage(void)
+static void usage()
 {
 	printf("Usage:\n"
 	       "  -d, --device <dev>   Use device <dev> instead of /dev/cec0\n"
@@ -186,6 +194,7 @@ static void usage(void)
 	       "  -t, --timeout <secs> Set the standby/resume timeout to <secs>. Default is 60s.\n"
 	       "\n"
 	       "  -A, --test-adapter                  Test the CEC adapter API\n"
+	       "  -F, --test-fuzzing                  Test by fuzzing CEC messages\n"
 	       "  --test-core                         Test the core functionality\n"
 	       "\n"
 	       "By changing --test to --skip-test in the following options you can skip tests\n"
@@ -213,6 +222,11 @@ static void usage(void)
 	       "                                      testing of Standby, Give Device Power Status and One Touch Play.\n"
 	       "\n"
 	       "  -E, --exit-on-fail Exit on the first fail.\n"
+	       "  -l, --list-tests   List all tests.\n"
+	       "  -e, --expect <test>=<result>\n"
+	       "                     Fail if the test gave a different result.\n"
+	       "  -N, --expect-with-no-warnings <test>=<result>\n"
+	       "                     Fail if the test gave a different result or if the test generated warnings.\n"
 	       "  -h, --help         Display this help message\n"
 	       "  -C, --color <when> Highlight OK/warn/fail/FAIL strings with colors\n"
 	       "                     <when> can be set to always, never, or auto (the default)\n"
@@ -225,6 +239,24 @@ static void usage(void)
 	       );
 }
 
+std::string safename(const char *name)
+{
+	std::string s;
+	bool not_alnum = false;
+
+	while (*name) {
+		if (isalnum(*name)) {
+			if (not_alnum && !s.empty())
+				s += '-';
+			s += tolower(*name);
+			not_alnum = false;
+		} else if (!not_alnum)
+			not_alnum = true;
+		name++;
+	}
+	return s;
+}
+
 static std::string ts2s(__u64 ts)
 {
 	std::string s;
@@ -306,7 +338,7 @@ static std::string audio_format_code2s(__u8 format_code)
 		return "Extended";
 	default:
 		return "Illegal";
-	};
+	}
 }
 
 std::string extension_type_code2s(__u8 type_code)
@@ -348,7 +380,7 @@ std::string short_audio_desc2s(const struct short_audio_desc &sad)
 		oss << audio_format_code2s(sad.format_code);
 	else
 		oss << extension_type_code2s(sad.extension_type_code);
-	oss << ", " << (int)sad.num_channels << " channels";
+	oss << ", " << static_cast<int>(sad.num_channels) << " channels";
 
 	oss << ", sampling rates (kHz): ";
 	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_32)
@@ -456,6 +488,7 @@ void sad_decode(struct short_audio_desc *sad, __u32 descriptor)
 		case SAD_EXT_TYPE_MPEG_H_3D_AUDIO:
 		case SAD_EXT_TYPE_AC_4:
 			sad->format_dependent = b3 & 0x07;
+			fallthrough;
 		case SAD_EXT_TYPE_LPCM_3D_AUDIO:
 			sad->bit_depth_mask = b3 & 0x07;
 			break;
@@ -590,26 +623,26 @@ std::string opcode2s(const struct cec_msg *msg)
 {
 	std::stringstream oss;
 	__u8 opcode = msg->msg[1];
+	const char *name;
 
 	if (msg->len == 1)
 		return "MSG_POLL";
 
 	if (opcode == CEC_MSG_CDC_MESSAGE) {
 		__u8 cdc_opcode = msg->msg[4];
+		name = cec_cdc_opcode2s(cdc_opcode);
 
-		for (unsigned i = 0; i < ARRAY_SIZE(cdcmsgtable); i++) {
-			if (cdcmsgtable[i].opcode == cdc_opcode)
-				return cdcmsgtable[i].name;
-		}
-		oss << "CDC: 0x" << std::hex << (unsigned)cdc_opcode;
+		if (name)
+			return name;
+		oss << "CDC: 0x" << std::hex << static_cast<unsigned>(cdc_opcode);
 		return oss.str();
 	}
 
-	for (unsigned i = 0; i < ARRAY_SIZE(msgtable); i++) {
-		if (msgtable[i].opcode == opcode)
-			return msgtable[i].name;
-	}
-	oss << "0x" << std::hex << (unsigned)opcode;
+	name = cec_opcode2s(opcode);
+
+	if (name)
+		return name;
+	oss << "0x" << std::hex << static_cast<unsigned>(opcode);
 	return oss.str();
 }
 
@@ -618,7 +651,7 @@ int cec_named_ioctl(struct node *node, const char *name,
 {
 	int retval;
 	int e;
-	struct cec_msg *msg = (struct cec_msg *)parm;
+	struct cec_msg *msg = static_cast<struct cec_msg *>(parm);
 	__u8 opcode = 0;
 	std::string opname;
 
@@ -679,7 +712,7 @@ int cec_named_ioctl(struct node *node, const char *name,
 		}
 		if ((msg->tx_status & ~CEC_TX_STATUS_OK) ||
 		    (msg->rx_status & ~CEC_RX_STATUS_OK))
-			printf("\t\t\tStatus: %s\n", status2s(*msg).c_str());
+			printf("\t\t\tStatus: %s\n", cec_status2s(*msg).c_str());
 		if (msg->tx_status & CEC_TX_STATUS_TIMEOUT)
 			warn("CEC_TX_STATUS_TIMEOUT was set, should not happen.\n");
 	}
@@ -726,37 +759,53 @@ int cec_named_ioctl(struct node *node, const char *name,
 	return retval == -1 ? e : (retval ? -1 : 0);
 }
 
+const char *result_name(int res, bool show_colors)
+{
+	switch (res) {
+	case OK_NOT_SUPPORTED:
+		return show_colors ? COLOR_GREEN("OK") " (Not Supported)" : "OK (Not Supported)";
+	case OK_PRESUMED:
+		return show_colors ? COLOR_GREEN("OK") " (Presumed)" : "OK (Presumed)";
+	case OK_REFUSED:
+		return show_colors ? COLOR_GREEN("OK") " (Refused)" : "OK (Refused)";
+	case OK_UNEXPECTED:
+		return show_colors ? COLOR_GREEN("OK") " (Unexpected)" : "OK (Unexpected)";
+	case OK_EXPECTED_FAIL:
+		return show_colors ? COLOR_GREEN("OK") " (Expected Failure)" : "OK (Expected Failure)";
+	case OK:
+		return show_colors ? COLOR_GREEN("OK") : "OK";
+	default:
+		return show_colors ? COLOR_RED("FAIL") : "FAIL";
+	}
+}
+
 const char *ok(int res)
 {
-	static char buf[100];
+	const char *res_name = result_name(res, show_colors);
 
-	if (res == NOTSUPPORTED) {
-		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Not Supported)" :
-		       "OK (Not Supported)");
-		res = 0;
-	} else if (res == PRESUMED_OK) {
-		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Presumed)" :
-		       "OK (Presumed)");
-		res = 0;
-	} else if (res == REFUSED) {
-		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Refused)" :
-		       "OK (Refused)");
-		res = 0;
-	} else
-		strcpy(buf, show_colors ? COLOR_GREEN("OK") : "OK");
-	tests_total++;
-	if (res) {
-		app_result = res;
-		sprintf(buf, show_colors ? COLOR_RED("FAIL") : "FAIL");
-	} else {
-		tests_ok++;
+	switch (res) {
+	case OK_NOT_SUPPORTED:
+	case OK_PRESUMED:
+	case OK_REFUSED:
+	case OK_UNEXPECTED:
+	case OK_EXPECTED_FAIL:
+	case OK:
+		res = OK;
+		break;
+	default:
+		break;
 	}
-	return buf;
+	tests_total++;
+	if (res)
+		app_result = res;
+	else
+		tests_ok++;
+	return res_name;
 }
 
 int check_0(const void *p, int len)
 {
-	const __u8 *q = (const __u8 *)p;
+	const __u8 *q = static_cast<const __u8 *>(p);
 
 	while (len--)
 		if (*q++)
@@ -830,7 +879,7 @@ retry:
 	res = doioctl(node, CEC_TRANSMIT, msg);
 	if (res == ENODEV) {
 		printf("Device was disconnected.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	if (res == ENONET) {
 		if (retried) {
@@ -890,6 +939,35 @@ retry:
 	return true;
 }
 
+int util_receive(struct node *node, unsigned la, unsigned timeout,
+		 struct cec_msg *msg, __u8 sent_msg, __u8 reply1, __u8 reply2)
+{
+	unsigned ts_start = get_ts_ms();
+
+	while (get_ts_ms() - ts_start < timeout) {
+		memset(msg, 0, sizeof(*msg));
+		msg->timeout = 20;
+		if (doioctl(node, CEC_RECEIVE, msg))
+			continue;
+		if (cec_msg_initiator(msg) != la)
+			continue;
+
+		if (msg->msg[1] == CEC_MSG_FEATURE_ABORT) {
+			__u8 reason, abort_msg;
+
+			cec_ops_feature_abort(msg, &abort_msg, &reason);
+			if (abort_msg != sent_msg)
+				continue;
+			return 0;
+		}
+
+		if (msg->msg[1] == reply1 || (reply2 && msg->msg[1] == reply2))
+			return msg->msg[1];
+	}
+
+	return -1;
+}
+
 static int poll_remote_devs(struct node *node)
 {
 	unsigned retries = 0;
@@ -912,7 +990,7 @@ static int poll_remote_devs(struct node *node)
 		} else {
 			if (!(msg.tx_status & CEC_TX_STATUS_ARB_LOST))
 				warn("retry poll due to unexpected status: %s\n",
-				     status2s(msg).c_str());
+				     cec_status2s(msg).c_str());
 			retries++;
 			fail_on_test(retries > 10);
 			i--;
@@ -924,17 +1002,17 @@ static int poll_remote_devs(struct node *node)
 static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 {
 	struct cec_msg msg = { };
-	bool ok;
+	bool unknown;
 
 	printf("\tSystem Information for device %d (%s) from device %d (%s):\n",
-	       i, la2s(i), la, la2s(la));
+	       i, cec_la2s(i), la, cec_la2s(la));
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_get_cec_version(&msg, true);
-	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
+	unknown = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tCEC Version                : ");
-	if (ok) {
-		printf("%s\n", status2s(msg).c_str());
+	if (unknown) {
+		printf("%s\n", cec_status2s(msg).c_str());
 		node->remote[i].cec_version = CEC_OP_CEC_VERSION_1_4;
 	}
 	/* This needs to be kept in sync with newer CEC versions */
@@ -949,15 +1027,15 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 			warn("The reported CEC version is greater than 2.0. The device will be tested as a CEC 2.0 compliant device.\n");
 		}
 		else
-			printf("%s\n", version2s(msg.msg[2]));
+			printf("%s\n", cec_version2s(msg.msg[2]));
 	}
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_give_physical_addr(&msg, true);
-	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
+	unknown = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tPhysical Address           : ");
-	if (ok) {
-		printf("%s\n", status2s(msg).c_str());
+	if (unknown) {
+		printf("%s\n", cec_status2s(msg).c_str());
 		node->remote[i].phys_addr = CEC_PHYS_ADDR_INVALID;
 	}
 	else {
@@ -966,29 +1044,29 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 		       cec_phys_addr_exp(node->remote[i].phys_addr));
 		node->remote[i].prim_type = msg.msg[4];
 		printf("\t\tPrimary Device Type        : %s\n",
-		       prim_type2s(node->remote[i].prim_type));
+		       cec_prim_type2s(node->remote[i].prim_type));
 	}
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_give_device_vendor_id(&msg, true);
-	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
+	unknown = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tVendor ID                  : ");
-	if (ok) {
-		printf("%s\n", status2s(msg).c_str());
+	if (unknown) {
+		printf("%s\n", cec_status2s(msg).c_str());
 		node->remote[i].vendor_id = CEC_VENDOR_ID_NONE;
 	} else {
 		node->remote[i].vendor_id = (msg.msg[2] << 16) |
 			(msg.msg[3] << 8) | msg.msg[4];
 		printf("0x%06x %s\n", node->remote[i].vendor_id,
-		       vendor2s(node->remote[i].vendor_id));
+		       cec_vendor2s(node->remote[i].vendor_id));
 	}
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_give_osd_name(&msg, true);
-	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
+	unknown = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tOSD Name                   : ");
-	if (ok) {
-		printf("%s\n", status2s(msg).c_str());
+	if (unknown) {
+		printf("%s\n", cec_status2s(msg).c_str());
 	} else {
 		cec_ops_set_osd_name(&msg, node->remote[i].osd_name);
 		printf("'%s'\n", node->remote[i].osd_name);
@@ -1004,10 +1082,10 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_give_device_power_status(&msg, true);
-	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
+	unknown = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tPower Status               : ");
-	if (ok) {
-		printf("%s\n", status2s(msg).c_str());
+	if (unknown) {
+		printf("%s\n", cec_status2s(msg).c_str());
 	} else {
 		__u8 pwr;
 
@@ -1066,6 +1144,7 @@ int main(int argc, char **argv)
 	int i;
 	const char *env_media_apps_color = getenv("MEDIA_APPS_COLOR");
 
+	srandom(time(NULL));
 	if (!env_media_apps_color || !strcmp(env_media_apps_color, "auto"))
 		show_colors = isatty(STDOUT_FILENO);
 	else if (!strcmp(env_media_apps_color, "always"))
@@ -1077,6 +1156,8 @@ int main(int argc, char **argv)
 			"cec-compliance: invalid value for MEDIA_APPS_COLOR environment variable\n");
 	}
 
+	collectTests();
+
 	for (i = 0; long_options[i].name; i++) {
 		if (!isalpha(long_options[i].val))
 			continue;
@@ -1088,7 +1169,7 @@ int main(int argc, char **argv)
 			short_options[idx++] = ':';
 		}
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -1097,7 +1178,7 @@ int main(int argc, char **argv)
 		if (ch == -1)
 			break;
 
-		options[(int)ch] = 1;
+		options[ch] = 1;
 		if (!option_index) {
 			for (i = 0; long_options[i].val; i++) {
 				if (long_options[i].val == ch) {
@@ -1144,7 +1225,7 @@ int main(int argc, char **argv)
 				show_colors = isatty(STDOUT_FILENO);
 			else {
 				usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			break;
 		case OptNoWarnings:
@@ -1156,6 +1237,14 @@ int main(int argc, char **argv)
 		case OptExitOnWarn:
 			exit_on_warn = true;
 			break;
+		case OptExpect:
+		case OptExpectWithNoWarnings:
+			if (setExpectedResult(optarg, ch == OptExpectWithNoWarnings)) {
+				fprintf(stderr, "invalid -e argument %s\n", optarg);
+				usage();
+				return 1;
+			}
+			break;
 		case OptRemote:
 			if (optarg) {
 				remote_la = strtoul(optarg, NULL, 0);
@@ -1197,7 +1286,7 @@ int main(int argc, char **argv)
 		if (device.empty()) {
 			fprintf(stderr,
 				"Could not find a CEC device for the given driver/adapter combination\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 	if (device.empty())
@@ -1206,7 +1295,7 @@ int main(int argc, char **argv)
 	if ((fd = open(device.c_str(), O_RDWR)) < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	struct node node = { };
@@ -1347,10 +1436,28 @@ int main(int argc, char **argv)
 
 	}
 
-	if (options[OptSkipInfo])
+	if (options[OptSkipInfo]) {
 		printf("\n");
-	else
-		cec_driver_info(caps, laddrs, node.phys_addr);
+	} else {
+		struct cec_connector_info conn_info = {};
+
+		doioctl(&node, CEC_ADAP_G_CONNECTOR_INFO, &conn_info);
+
+		cec_driver_info(caps, laddrs, node.phys_addr, conn_info);
+	}
+
+	if (options[OptListTests]) {
+		printf("\nAvailable Tests:\n\n");
+		listTests();
+		printf("\n");
+		printf("Possible test results:\n"
+		       "\t0 = OK                  Supported correctly by the device.\n"
+		       "\t1 = FAIL                Failed and was expected to be supported by this device.\n"
+		       "\t2 = OK (Presumed)       Presumably supported.  Manually check to confirm.\n"
+		       "\t3 = OK (Not Supported)  Not supported and not mandatory for the device.\n"
+		       "\t4 = OK (Refused)        Supported by the device, but was refused.\n"
+		       "\t5 = OK (Unexpected)     Supported correctly but is not expected to be supported for this device.\n");
+	}
 
 	bool missing_pa = node.phys_addr == CEC_PHYS_ADDR_INVALID && (node.caps & CEC_CAP_PHYS_ADDR);
 	bool missing_la = laddrs.num_log_addrs == 0 && (node.caps & CEC_CAP_LOG_ADDRS);
@@ -1362,18 +1469,19 @@ int main(int argc, char **argv)
 	if (missing_la)
 		fprintf(stderr, "FAIL: missing logical address(es), use cec-ctl to configure this\n");
 	if (missing_la || missing_pa)
-		exit(-1);
+		std::exit(EXIT_FAILURE);
 
 	if (!options[OptSkipInfo]) {
 		printf("\nCompliance test for %s device %s:\n\n",
 		       caps.driver, device.c_str());
 		printf("    The test results mean the following:\n"
-		       "        OK                  Supported correctly by the device.\n"
-		       "        OK (Not Supported)  Not supported and not mandatory for the device.\n"
-		       "        OK (Presumed)       Presumably supported.  Manually check to confirm.\n"
-		       "        OK (Unexpected)     Supported correctly but is not expected to be supported for this device.\n"
-		       "        OK (Refused)        Supported by the device, but was refused.\n"
-		       "        FAIL                Failed and was expected to be supported by this device.\n\n");
+		       "        OK                    Supported correctly by the device.\n"
+		       "        OK (Not Supported)    Not supported and not mandatory for the device.\n"
+		       "        OK (Presumed)         Presumably supported.  Manually check to confirm.\n"
+		       "        OK (Unexpected)       Supported correctly but is not expected to be supported for this device.\n"
+		       "        OK (Refused)          Supported by the device, but was refused.\n"
+		       "        OK (Expected Failure) Failed but this was expected (see -e option).\n"
+		       "        FAIL                  Failed and was expected to be supported by this device.\n\n");
 	}
 
 	node.has_cec20 = laddrs.cec_version >= CEC_OP_CEC_VERSION_2_0;
@@ -1386,7 +1494,7 @@ int main(int argc, char **argv)
 
 	if (!node.remote_la_mask) {
 		printf("\nFAIL: No remote devices found, exiting.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	if (options[OptTestAdapter])
@@ -1399,6 +1507,9 @@ int main(int argc, char **argv)
 			topology_probe_device(&node, i, node.log_addr[0]);
 	printf("\n");
 
+	if (options[OptTestFuzzing] && remote_la >= 0)
+		std::exit(testFuzzing(node, laddrs.log_addr[0], remote_la));
+
 	unsigned remote_la_mask = node.remote_la_mask;
 
 	if (remote_la >= 0)
@@ -1423,5 +1534,5 @@ int main(int argc, char **argv)
 	printf("Total for %s device %s: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
 	       caps.driver, device.c_str(),
 	       tests_total, tests_ok, tests_total - tests_ok, warnings);
-	exit(app_result);
+	std::exit(app_result);
 }
