@@ -17,6 +17,7 @@
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <sstream>
+#include <cstdlib>
 
 #include "cec-compliance.h"
 
@@ -39,12 +40,12 @@ static int flush_pending_msgs(struct node *node)
 	return 0;
 }
 
-int testCap(struct node *node)
+static int testCap(struct node *node)
 {
 	struct cec_caps caps;
 
 	memset(&caps, 0xff, sizeof(caps));
-	// Must always be there
+	fail_on_test(doioctl(node, CEC_ADAP_G_CAPS, NULL) != EFAULT);
 	fail_on_test(doioctl(node, CEC_ADAP_G_CAPS, &caps));
 	fail_on_test(caps.available_log_addrs == 0 ||
 		     caps.available_log_addrs > CEC_MAX_LOG_ADDRS);
@@ -53,7 +54,28 @@ int testCap(struct node *node)
 	return 0;
 }
 
-int testDQEvent(struct node *node)
+static int testInvalidIoctls(struct node *node)
+{
+	const char type = 'a';
+	unsigned ioc = _IOC(_IOC_NONE, type, 0xff, 0);
+	unsigned char buf[0x4000] = {};
+
+	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	ioc = _IOC(_IOC_NONE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	ioc = _IOC(_IOC_READ, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	ioc = _IOC(_IOC_WRITE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	ioc = _IOC(_IOC_READ | _IOC_WRITE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	return 0;
+}
+
+static int testDQEvent(struct node *node)
 {
 	struct cec_event ev;
 
@@ -69,7 +91,7 @@ int testDQEvent(struct node *node)
 	return 0;
 }
 
-int testAdapPhysAddr(struct node *node)
+static int testAdapPhysAddr(struct node *node)
 {
 	__u16 old_pa = 0xefff;
 	__u16 pa = 0x1000;
@@ -90,7 +112,7 @@ int testAdapPhysAddr(struct node *node)
 	return 0;
 }
 
-int testAdapLogAddrs(struct node *node)
+static int testAdapLogAddrs(struct node *node)
 {
 	static const __u8 la_types[] = {
 		CEC_LOG_ADDR_TYPE_TV,
@@ -244,7 +266,7 @@ int testAdapLogAddrs(struct node *node)
 	return 0;
 }
 
-int testTransmit(struct node *node)
+static int testTransmit(struct node *node)
 {
 	struct cec_msg msg = { };
 	unsigned i, la = node->log_addr[0];
@@ -256,7 +278,7 @@ int testTransmit(struct node *node)
 	if (!(node->caps & CEC_CAP_TRANSMIT)) {
 		cec_msg_init(&msg, la, 0);
 		fail_on_test(doioctl(node, CEC_TRANSMIT, &msg) != ENOTTY);
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	}
 
 	/* Check invalid messages */
@@ -437,7 +459,7 @@ int testTransmit(struct node *node)
 		time_t last_t = cur_t + 7;
 		unsigned max_cnt = 0;
 		unsigned cnt = 0;
-	
+
 		do {
 			t = time(NULL);
 			if (t != cur_t) {
@@ -461,7 +483,7 @@ int testTransmit(struct node *node)
 		time_t last_t = cur_t + 7;
 		unsigned max_cnt = 0;
 		unsigned cnt = 0;
-	
+
 		do {
 			t = time(NULL);
 			if (t != cur_t) {
@@ -483,14 +505,14 @@ int testTransmit(struct node *node)
 	return 0;
 }
 
-int testReceive(struct node *node)
+static int testReceive(struct node *node)
 {
 	unsigned la = node->log_addr[0], remote_la = 0;
 	struct cec_msg msg;
 
 	if (!(node->caps & CEC_CAP_TRANSMIT)) {
 		fail_on_test(doioctl(node, CEC_RECEIVE, &msg) != ENOTTY);
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	}
 
 	for (unsigned i = 0; i < 15; i++) {
@@ -502,7 +524,7 @@ int testReceive(struct node *node)
 	fail_on_test(flush_pending_msgs(node));
 
 	if (remote_la == 15)
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 
 	cec_msg_init(&msg, la, remote_la);
 	cec_msg_give_physical_addr(&msg, false);
@@ -544,13 +566,13 @@ int testReceive(struct node *node)
 	return 0;
 }
 
-int testNonBlocking(struct node *node)
+static int testNonBlocking(struct node *node)
 {
 	unsigned la = node->log_addr[0], remote_la = 0, invalid_remote = 0xf;
 	struct cec_msg msg;
 
 	if (!(node->caps & CEC_CAP_TRANSMIT))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 
 	for (unsigned i = 0; i < 15; i++) {
 		if (node->remote_la_mask & (1 << i))
@@ -794,11 +816,11 @@ int testNonBlocking(struct node *node)
 		fail_on_test(doioctl(node, CEC_S_MODE, &mode));
 	}
 	if (remote_la == 15 || invalid_remote == 15)
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	return 0;
 }
 
-int testModes(struct node *node, struct node *node2)
+static int testModes(struct node *node, struct node *node2)
 {
 	struct cec_msg msg;
 	__u8 me = node->log_addr[0];
@@ -997,7 +1019,7 @@ static void print_sfts(unsigned sft[12], const char *descr)
 		printf("\t\t%s: %s\n", descr, s.c_str());
 }
 
-int testLostMsgs(struct node *node)
+static int testLostMsgs(struct node *node)
 {
 	struct cec_msg msg;
 	struct cec_event ev;
@@ -1214,6 +1236,7 @@ void testAdapter(struct node &node, struct cec_log_addrs &laddrs,
 
 	printf("\nCEC API:\n");
 	printf("\tCEC_ADAP_G_CAPS: %s\n", ok(testCap(&node)));
+	printf("\tInvalid ioctls: %s\n", ok(testInvalidIoctls(&node)));
 	printf("\tCEC_DQEVENT: %s\n", ok(testDQEvent(&node)));
 	printf("\tCEC_ADAP_G/S_PHYS_ADDR: %s\n", ok(testAdapPhysAddr(&node)));
 	if (node.caps & CEC_CAP_PHYS_ADDR)
@@ -1221,7 +1244,7 @@ void testAdapter(struct node &node, struct cec_log_addrs &laddrs,
 	if (node.phys_addr == CEC_PHYS_ADDR_INVALID) {
 		fprintf(stderr, "FAIL: without a valid physical address this test cannot proceed.\n");
 		fprintf(stderr, "Make sure that this CEC adapter is connected to another HDMI sink or source.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	printf("\tCEC_ADAP_G/S_LOG_ADDRS: %s\n", ok(testAdapLogAddrs(&node)));
 	fcntl(node.fd, F_SETFL, fcntl(node.fd, F_GETFL) & ~O_NONBLOCK);
@@ -1253,7 +1276,7 @@ void testAdapter(struct node &node, struct cec_log_addrs &laddrs,
 	if ((node2.fd = open(device, O_RDWR)) < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device,
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	printf("\tCEC_G/S_MODE: %s\n", ok(testModes(&node, &node2)));
