@@ -15,6 +15,7 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <config.h>
+#include <cstdlib>
 
 #include "cec-compliance.h"
 
@@ -31,7 +32,7 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 	fail_on_test_v2(node->remote[la].cec_version,
 			timed_out(&msg) && is_tv(la, node->remote[la].prim_type));
 	if (timed_out(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 
 	/* When the device supports Dynamic Auto Lipsync but does not implement
 	   CEC 1.4b, or 2.0, a very strict subset of CEC can be supported. If we
@@ -54,8 +55,17 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 	if (audio_out_compensated == CEC_OP_AUD_OUT_COMPENSATED_PARTIAL_DELAY) {
 		info("Audio out delay: %d (%dms)\n", audio_out_delay, (audio_out_delay - 1) * 2);
 		fail_on_test(audio_out_delay == 0 || audio_out_delay > 251);
+		// Warn if the delay is more than 50 ms
+		warn_on_test(audio_out_delay > (50 / 2) + 1);
 	}
 	fail_on_test(video_latency == 0 || video_latency > 251);
+	// Warn if the delay is more than 50 ms and low latency mode is set
+	if (video_latency > (50 / 2) + 1) {
+		if (low_latency_mode)
+			fail("Low latency mode is set, but video latency is > 50ms\n");
+		else
+			warn("Video latency is > 50ms\n");
+	}
 
 	return 0;
 }
@@ -104,9 +114,7 @@ static bool pa_are_adjacent(__u16 pa1, __u16 pa2)
 		return false;
 	if ((pa1 & trail_mask) || (pa2 & trail_mask))
 		return false;
-	if (!((pa1 & ~mask) && (pa2 & ~mask)))
-		return true;
-	return false;
+	return !((pa1 & ~mask) && (pa2 & ~mask));
 }
 
 static bool pa_is_upstream_from(__u16 pa1, __u16 pa2)
@@ -115,36 +123,7 @@ static bool pa_is_upstream_from(__u16 pa1, __u16 pa2)
 
 	if (pa1 == CEC_PHYS_ADDR_INVALID || pa2 == CEC_PHYS_ADDR_INVALID)
 		return false;
-	if (!(pa1 & ~mask) && (pa2 & ~mask))
-		return true;
-	return false;
-}
-
-static bool util_receive(struct node *node, unsigned la, struct cec_msg *msg,
-			 __u8 sent_msg, __u8 reply1, __u8 reply2 = 0)
-{
-        while (1) {
-		memset(msg, 0, sizeof(*msg));
-		msg->timeout = 1;
-		if (doioctl(node, CEC_RECEIVE, msg))
-			break;
-		if (cec_msg_initiator(msg) != la)
-			continue;
-
-		if (msg->msg[1] == CEC_MSG_FEATURE_ABORT) {
-			__u8 reason, abort_msg;
-
-			cec_ops_feature_abort(msg, &abort_msg, &reason);
-			if (abort_msg != sent_msg)
-				continue;
-			return true;
-		}
-
-		if (msg->msg[1] == reply1 || (reply2 && msg->msg[1] == reply2))
-			return true;
-	}
-
-	return false;
+	return !(pa1 & ~mask) && (pa2 & ~mask);
 }
 
 static int arc_initiate_tx(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -167,11 +146,11 @@ static int arc_initiate_tx(struct node *node, unsigned me, unsigned la, bool int
 	if (timed_out(&msg)) {
 		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].sink_has_arc_tx);
 		warn("Timed out waiting for Report ARC Initiated/Terminated.\n");
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	}
 	if (unrecognized_op(&msg)) {
 		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].sink_has_arc_tx);
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	}
 	if (cec_msg_opcode(&msg) == CEC_MSG_REPORT_ARC_INITIATED) {
 		fail_on_test(!pa_are_adjacent(node->phys_addr, node->remote[la].phys_addr));
@@ -181,9 +160,9 @@ static int arc_initiate_tx(struct node *node, unsigned me, unsigned la, bool int
 	else if (cec_msg_opcode(&msg) == CEC_MSG_REPORT_ARC_TERMINATED)
 		announce("Device supports ARC but is not ready to initiate.");
 	else if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	else if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 
 	return 0;
 }
@@ -204,14 +183,14 @@ static int arc_terminate_tx(struct node *node, unsigned me, unsigned la, bool in
 	fail_on_test(!transmit_timeout(node, &msg));
 	if (timed_out(&msg)) {
 		warn("Timed out waiting for Report ARC Terminated.\n");
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	}
 	fail_on_test(unrecognized_op(&msg));
 	if (cec_msg_status_is_abort(&msg)) {
 		warn("Received Feature Abort for Terminate ARC (but the message was recognized).\n");
 		if (refused(&msg))
-			return REFUSED;
-		return PRESUMED_OK;
+			return OK_REFUSED;
+		return OK_PRESUMED;
 	}
 
 	return 0;
@@ -242,14 +221,14 @@ static int arc_initiate_rx(struct node *node, unsigned me, unsigned la, bool int
 			announce("The device supports ARC but is not ready to initiate.");
 			return 0;
 		}
-		else {
-			warn("Device responded Feature Abort with unexpected abort reason. Assuming no ARC support.\n");
-			unsupported = true;
-		}
+
+		warn("Device responded Feature Abort with unexpected abort reason. Assuming no ARC support.\n");
+		unsupported = true;
 	}
+
 	if (unsupported) {
 		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].source_has_arc_rx);
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	}
 	fail_on_test(!pa_are_adjacent(node->phys_addr, node->remote[la].phys_addr));
 	fail_on_test_v2(node->remote[la].cec_version, !node->remote[la].source_has_arc_rx);
@@ -279,14 +258,14 @@ static int arc_terminate_rx(struct node *node, unsigned me, unsigned la, bool in
 	fail_on_test(!transmit_timeout(node, &msg));
 	if (timed_out(&msg)) {
 		warn("Timed out waiting for Terminate ARC.\n");
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	}
 	fail_on_test(unrecognized_op(&msg));
 	if (cec_msg_status_is_abort(&msg)) {
 		warn("Received Feature Abort for Request ARC Termination (but the message was recognized).\n");
 		if (refused(&msg))
-			return REFUSED;
-		return PRESUMED_OK;
+			return OK_REFUSED;
+		return OK_PRESUMED;
 	}
 
 	cec_msg_init(&msg, me, la);
@@ -358,11 +337,11 @@ static int sac_request_sad_probe(struct node *node, unsigned me, unsigned la, bo
 	fail_on_test(!transmit_timeout(node, &msg));
 	fail_on_test(timed_out(&msg));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	node->remote[la].has_sad = true;
 
 	return 0;
@@ -479,12 +458,12 @@ static int sac_set_system_audio_mode_direct(struct node *node, unsigned me, unsi
 	fail_on_test_v2(node->remote[la].cec_version,
 			unrecognized_op(&msg) && is_tv(la, node->remote[la].prim_type));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	node->remote[la].has_set_sys_audio_mode = true;
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 static int sac_set_system_audio_mode_broadcast_on(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -495,7 +474,7 @@ static int sac_set_system_audio_mode_broadcast_on(struct node *node, unsigned me
 	cec_msg_set_system_audio_mode(&msg, CEC_OP_SYS_AUD_STATUS_ON);
 	fail_on_test(!transmit_timeout(node, &msg));
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 static int sac_system_audio_mode_status(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -512,12 +491,12 @@ static int sac_system_audio_mode_status(struct node *node, unsigned me, unsigned
 	fail_on_test_v2(node->remote[la].cec_version,
 			is_tv(la, node->remote[la].prim_type) && unrecognized_op(&msg));
 	if (unrecognized_op(&msg) && !node->remote[la].has_set_sys_audio_mode)
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	fail_on_test(unrecognized_op(&msg));
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 
 	return 0;
 }
@@ -530,7 +509,7 @@ static int sac_set_system_audio_mode_broadcast_off(struct node *node, unsigned m
 	cec_msg_set_system_audio_mode(&msg, CEC_OP_SYS_AUD_STATUS_OFF);
 	fail_on_test(!transmit_timeout(node, &msg));
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 static int sac_system_audio_mode_req_on(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -550,11 +529,11 @@ static int sac_system_audio_mode_req_on(struct node *node, unsigned me, unsigned
 	fail_on_test_v2(node->remote[la].cec_version,
 			cec_has_audiosystem(1 << la) && unrecognized_op(&msg));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	node->remote[la].has_sys_audio_mode_req = true;
 	cec_ops_set_system_audio_mode(&msg, &status);
 	fail_on_test(status != CEC_OP_SYS_AUD_STATUS_ON);
@@ -578,12 +557,12 @@ static int sac_give_system_audio_mode_status(struct node *node, unsigned me, uns
 	fail_on_test_v2(node->remote[la].cec_version,
 			cec_has_audiosystem(1 << la) && unrecognized_op(&msg));
 	if (unrecognized_op(&msg) && !node->remote[la].has_sys_audio_mode_req)
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	fail_on_test(unrecognized_op(&msg));
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	cec_ops_system_audio_mode_status(&msg, &system_audio_status);
 	fail_on_test(system_audio_status != CEC_OP_SYS_AUD_STATUS_ON);
 
@@ -603,11 +582,11 @@ static int sac_give_audio_status(struct node *node, unsigned me, unsigned la, bo
 	fail_on_test_v2(node->remote[la].cec_version,
 			cec_has_audiosystem(1 << la) && unrecognized_op(&msg));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 
 	cec_ops_report_audio_status(&msg, &node->remote[la].mute, &node->remote[la].volume);
 	fail_on_test(node->remote[la].volume > 100);
@@ -634,11 +613,10 @@ static int sac_util_send_user_control_press(struct node *node, unsigned me, unsi
 	   It is recommended for devices to not send Report Audio Status back
 	   more often than once every 500ms. We therefore sleep a second before
 	   each User Control Pressed is sent. */
-	__u32 mode = CEC_MODE_INITIATOR | CEC_MODE_FOLLOWER;
 	bool got_response;
 
 	sleep(1);
-	doioctl(node, CEC_S_MODE, &mode);
+	mode_set_follower(node);
 	cec_msg_init(&msg, me, la);
 	rc_press.ui_cmd = ui_cmd;
 	cec_msg_user_control_pressed(&msg, &rc_press);
@@ -646,27 +624,26 @@ static int sac_util_send_user_control_press(struct node *node, unsigned me, unsi
 	cec_msg_init(&msg, me, la);
 	cec_msg_user_control_released(&msg);
 	fail_on_test(!transmit_timeout(node, &msg));
-	got_response = util_receive(node, la, &msg, CEC_MSG_USER_CONTROL_PRESSED, CEC_MSG_REPORT_AUDIO_STATUS);
-	mode = CEC_MODE_INITIATOR;
-	doioctl(node, CEC_S_MODE, &mode);
+	got_response = util_receive(node, la, 1000, &msg,
+				    CEC_MSG_USER_CONTROL_PRESSED, CEC_MSG_REPORT_AUDIO_STATUS) >= 0;
 
 	fail_on_test_v2(node->remote[la].cec_version, !got_response &&
 			cec_has_audiosystem(1 << la));
 	fail_on_test_v2(node->remote[la].cec_version, unrecognized_op(&msg) &&
 			(is_tv(la, node->remote[la].prim_type) || cec_has_audiosystem(1 << la)));
 	if (unrecognized_op(&msg) && !node->remote[la].has_sys_audio_mode_req)
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	fail_on_test(unrecognized_op(&msg));
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	if (got_response) {
 		cec_ops_report_audio_status(&msg, &node->remote[la].mute, &node->remote[la].volume);
 		return 0;
 	}
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 static int sac_user_control_press_vol_up(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -751,14 +728,14 @@ static int sac_user_control_release(struct node *node, unsigned me, unsigned la,
 	fail_on_test_v2(node->remote[la].cec_version, unrecognized_op(&msg) &&
 			(is_tv(la, node->remote[la].prim_type) || cec_has_audiosystem(1 << la)));
 	if (unrecognized_op(&msg) && !node->remote[la].has_sys_audio_mode_req)
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	fail_on_test(unrecognized_op(&msg));
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 static int sac_system_audio_mode_req_off(struct node *node, unsigned me, unsigned la, bool interactive)
@@ -776,11 +753,11 @@ static int sac_system_audio_mode_req_off(struct node *node, unsigned me, unsigne
 	fail_on_test_v2(node->remote[la].cec_version,
 			cec_has_audiosystem(1 << la) && unrecognized_op(&msg));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 	if (cec_msg_status_is_abort(&msg))
-		return PRESUMED_OK;
+		return OK_PRESUMED;
 	cec_ops_set_system_audio_mode(&msg, &status);
 	fail_on_test(status != CEC_OP_SYS_AUD_STATUS_OFF);
 
@@ -833,7 +810,7 @@ struct remote_subtest sac_subtests[] = {
 	{ "User Control Pressed (Mute Function)",
 	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
 	  sac_user_control_press_mute_function },
-	{ "User Control Released",
+	{ "User Control Released (Audio)",
 	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
 	  sac_user_control_release },
 	{ "Set System Audio Mode (broadcast off)",
@@ -866,11 +843,11 @@ static int audio_rate_ctl_set_audio_rate(struct node *node, unsigned me, unsigne
 	fail_on_test_v2(node->remote[la].cec_version,
 			!node->remote[la].has_aud_rate && !unrecognized_op(&msg));
 	if (unrecognized_op(&msg))
-		return NOTSUPPORTED;
+		return OK_NOT_SUPPORTED;
 	if (refused(&msg))
-		return REFUSED;
+		return OK_REFUSED;
 
-	return PRESUMED_OK;
+	return OK_PRESUMED;
 }
 
 struct remote_subtest audio_rate_ctl_subtests[] = {
