@@ -3,10 +3,12 @@
  * Copyright 2016 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  */
 
+#include <cstdlib>
+#include <cstring>
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 #include <inttypes.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -19,12 +21,11 @@
 #include <sstream>
 
 #include "cec-follower.h"
+#include "compiler.h"
 #ifndef ANDROID
 #include "version.h"
 #endif
 
-#include "cec-table.h"
-
 /* Short option list
 
    Please keep in alphabetical order.
@@ -37,13 +38,15 @@ enum Option {
 	OptSetDevice = 'd',
 	OptSetDriver = 'D',
 	OptHelp = 'h',
+	OptIgnore = 'i',
 	OptNoWarnings = 'n',
 	OptTrace = 'T',
 	OptVerbose = 'v',
 	OptShowMsgs = 'm',
 	OptShowState = 's',
 	OptWallClock = 'w',
-	OptLast = 128
+	OptServiceByDigID = 128,
+	OptLast = 256
 };
 
 static char options[OptLast];
@@ -55,21 +58,23 @@ bool show_warnings = true;
 unsigned warnings;
 
 static struct option long_options[] = {
-	{"device", required_argument, 0, OptSetDevice},
-	{"adapter", required_argument, 0, OptSetAdapter},
-	{"driver", required_argument, 0, OptSetDriver},
-	{"help", no_argument, 0, OptHelp},
-	{"no-warnings", no_argument, 0, OptNoWarnings},
-	{"trace", no_argument, 0, OptTrace},
-	{"verbose", no_argument, 0, OptVerbose},
-	{"show-msgs", no_argument, 0, OptShowMsgs},
-	{"show-state", no_argument, 0, OptShowState},
-	{"wall-clock", no_argument, 0, OptWallClock},
+	{ "device", required_argument, 0, OptSetDevice },
+	{ "adapter", required_argument, 0, OptSetAdapter },
+	{ "driver", required_argument, 0, OptSetDriver },
+	{ "help", no_argument, 0, OptHelp },
+	{ "no-warnings", no_argument, 0, OptNoWarnings },
+	{ "trace", no_argument, 0, OptTrace },
+	{ "verbose", no_argument, 0, OptVerbose },
+	{ "show-msgs", no_argument, 0, OptShowMsgs },
+	{ "show-state", no_argument, 0, OptShowState },
+	{ "wall-clock", no_argument, 0, OptWallClock },
+	{ "service-by-dig-id", no_argument, 0, OptServiceByDigID },
+	{ "ignore", required_argument, 0, OptIgnore },
 
-	{0, 0, 0, 0}
+	{ 0, 0, 0, 0 }
 };
 
-static void usage(void)
+static void usage()
 {
 	printf("Usage:\n"
 	       "  -d, --device <dev>  Use device <dev> instead of /dev/cec0\n"
@@ -83,6 +88,11 @@ static void usage(void)
 	       "  -w, --wall-clock    Show timestamps as wall-clock time (implies -v)\n"
 	       "  -m, --show-msgs     Show received messages\n"
 	       "  -s, --show-state    Show state changes from the emulated device\n"
+	       "  --service-by-dig-id Report digital services by digital ID instead of by channel\n"
+	       "  -i, --ignore <la>,<opcode>\n"
+	       "                      Ignore messages from logical address <la> and opcode\n"
+	       "                      <opcode>. 'all' can be used for <la> or <opcode> to match\n"
+	       "                      all logical addresses or opcodes.\n"
 	       );
 }
 
@@ -135,6 +145,7 @@ void sad_encode(const struct short_audio_desc *sad, __u32 *descriptor)
 		case SAD_EXT_TYPE_MPEG_H_3D_AUDIO:
 		case SAD_EXT_TYPE_AC_4:
 			b3 |= sad->format_dependent & 0x07;
+			fallthrough;
 		case SAD_EXT_TYPE_LPCM_3D_AUDIO:
 			b3 |= sad->bit_depth_mask & 0x07;
 			break;
@@ -182,7 +193,7 @@ static std::string audio_format_code2s(__u8 format_code)
 		return "Extended";
 	default:
 		return "Illegal";
-	};
+	}
 }
 
 std::string extension_type_code2s(__u8 type_code)
@@ -220,7 +231,7 @@ std::string audio_format_id_code2s(__u8 audio_format_id, __u8 audio_format_code)
 {
 	if (audio_format_id == 0)
 		return audio_format_code2s(audio_format_code);
-	else if (audio_format_id == 1)
+	if (audio_format_id == 1)
 		return extension_type_code2s(audio_format_code);
 	return "Invalid";
 }
@@ -229,23 +240,23 @@ std::string opcode2s(const struct cec_msg *msg)
 {
 	std::stringstream oss;
 	__u8 opcode = msg->msg[1];
+	const char *name;
 
 	if (opcode == CEC_MSG_CDC_MESSAGE) {
 		__u8 cdc_opcode = msg->msg[4];
+		name = cec_cdc_opcode2s(cdc_opcode);
 
-		for (unsigned i = 0; i < ARRAY_SIZE(cdcmsgtable); i++) {
-			if (cdcmsgtable[i].opcode == cdc_opcode)
-				return cdcmsgtable[i].name;
-		}
-		oss << "CDC: 0x" << std::hex << (unsigned)cdc_opcode;
+		if (name)
+			return name;
+		oss << "CDC: 0x" << std::hex << static_cast<unsigned>(cdc_opcode);
 		return oss.str();
 	}
 
-	for (unsigned i = 0; i < ARRAY_SIZE(msgtable); i++) {
-		if (msgtable[i].opcode == opcode)
-			return msgtable[i].name;
-	}
-	oss << "0x" << std::hex << (unsigned)opcode;
+	name = cec_opcode2s(opcode);
+
+	if (name)
+		return name;
+	oss << "0x" << std::hex << static_cast<unsigned>(opcode);
 	return oss.str();
 }
 
@@ -263,7 +274,7 @@ int cec_named_ioctl(int fd, const char *name,
 			name, retval, strerror(e));
 
 	if (!retval) {
-		const struct cec_msg *msg = (const struct cec_msg *)parm;
+		const struct cec_msg *msg = static_cast<const struct cec_msg *>(parm);
 
 		/* Update the timestamp whenever we successfully transmit to an LA,
 		   or whenever we receive something from the LA */
@@ -298,11 +309,13 @@ void state_init(struct node &node)
 	node.state.sac_active = false;
 	node.state.volume = 50;
 	node.state.mute = false;
+	tuner_dev_info_init(&node.state);
 }
 
 int main(int argc, char **argv)
 {
 	std::string device;
+	struct node node = { };
 	const char *driver = NULL;
 	const char *adapter = NULL;
 	char short_options[26 * 2 * 2 + 1];
@@ -322,7 +335,7 @@ int main(int argc, char **argv)
 			short_options[idx++] = ':';
 		}
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -331,7 +344,7 @@ int main(int argc, char **argv)
 		if (ch == -1)
 			break;
 
-		options[(int)ch] = 1;
+		options[ch] = 1;
 		if (!option_index) {
 			for (i = 0; long_options[i].val; i++) {
 				if (long_options[i].val == ch) {
@@ -372,6 +385,42 @@ int main(int argc, char **argv)
 		case OptShowState:
 			show_state = true;
 			break;
+		case OptIgnore: {
+			bool all_la = !strncmp(optarg, "all", 3);
+			bool all_opcodes = true;
+			const char *sep = std::strchr(optarg, ',');
+			unsigned la_mask = 0xffff, opcode, la = 0;
+
+			if (sep)
+				all_opcodes = !strncmp(sep + 1, "all", 3);
+			if (!all_la) {
+				la = strtoul(optarg, NULL, 0);
+
+				if (la > 15) {
+					fprintf(stderr, "invalid logical address (> 15)\n");
+					usage();
+					return 1;
+				}
+				la_mask = 1 << la;
+			}
+			if (!all_opcodes) {
+				opcode = strtoul(sep + 1, NULL, 0);
+				if (opcode > 255) {
+					fprintf(stderr, "invalid opcode (> 255)\n");
+					usage();
+					return 1;
+				}
+				node.ignore_opcode[opcode] |= la_mask;
+				break;
+			}
+			if (all_la && all_opcodes) {
+				fprintf(stderr, "all,all is invalid\n");
+				usage();
+				return 1;
+			}
+			node.ignore_la[la] = true;
+			break;
+		}
 		case OptWallClock:
 		case OptVerbose:
 			show_info = true;
@@ -404,7 +453,7 @@ int main(int argc, char **argv)
 		if (device.empty()) {
 			fprintf(stderr,
 				"Could not find a CEC device for the given driver/adapter combination\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 	if (device.empty())
@@ -413,10 +462,9 @@ int main(int argc, char **argv)
 	if ((fd = open(device.c_str(), O_RDWR)) < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
-	struct node node = { };
 	struct cec_caps caps = { };
 
 	node.fd = fd;
@@ -424,6 +472,7 @@ int main(int argc, char **argv)
 	doioctl(&node, CEC_ADAP_G_CAPS, &caps);
 	node.caps = caps.capabilities;
 	node.available_log_addrs = caps.available_log_addrs;
+	node.state.service_by_dig_id = options[OptServiceByDigID];
 	state_init(node);
 
 #ifdef SHA
@@ -442,7 +491,13 @@ int main(int argc, char **argv)
 	node.cec_version = laddrs.cec_version;
 
 	printf("\n");
-	cec_driver_info(caps, laddrs, node.phys_addr);
+
+	struct cec_connector_info conn_info = {};
+
+	doioctl(&node, CEC_ADAP_G_CONNECTOR_INFO, &conn_info);
+
+	cec_driver_info(caps, laddrs, node.phys_addr, conn_info);
+
 	if (laddrs.cec_version >= CEC_OP_CEC_VERSION_2_0) {
 		bool is_dev_feat = false;
 
@@ -468,7 +523,9 @@ int main(int argc, char **argv)
 	}
 	printf("\n");
 
-	bool missing_pa = node.phys_addr == CEC_PHYS_ADDR_INVALID && (node.caps & CEC_CAP_PHYS_ADDR);
+	bool missing_pa = node.phys_addr == CEC_PHYS_ADDR_INVALID &&
+		(node.caps & CEC_CAP_PHYS_ADDR) &&
+		!(node.caps & CEC_CAP_CONNECTOR_INFO);
 	bool missing_la = laddrs.num_log_addrs == 0 && (node.caps & CEC_CAP_LOG_ADDRS);
 
 	if (missing_la || missing_pa)
@@ -478,7 +535,7 @@ int main(int argc, char **argv)
 	if (missing_la)
 		fprintf(stderr, "FAIL: missing logical address(es), use cec-ctl to configure this\n");
 	if (missing_la || missing_pa)
-		exit(-1);
+		std::exit(EXIT_FAILURE);
 
 	testProcessing(&node, options[OptWallClock]);
 }
