@@ -22,6 +22,7 @@
 #endif
 
 #include <cec-info.h>
+#include <cec-log.h>
 
 #define ARRAY_SIZE(a) \
 	(sizeof(a) / sizeof(*a))
@@ -51,6 +52,11 @@ struct state {
 	__u64 rc_press_rx_ts;
 	unsigned rc_press_hold_count;
 	unsigned rc_duration_sum;
+	struct cec_op_tuner_device_info tuner_dev_info;
+	unsigned int service_idx;
+	bool service_by_dig_id;
+	bool tuner_report_changes;
+	bool deck_report_changes;
 };
 
 struct node {
@@ -70,6 +76,9 @@ struct node {
 	bool has_deck_ctl;
 	bool has_rec_tv;
 	bool has_osd_string;
+
+	bool ignore_la[16];
+	unsigned short ignore_opcode[256];
 };
 
 struct la_info {
@@ -144,9 +153,6 @@ struct short_audio_desc {
 #define SAD_EXT_TYPE_AC_4			12
 #define SAD_EXT_TYPE_LPCM_3D_AUDIO		13
 
-#define cec_phys_addr_exp(pa) \
-	((pa) >> 12), ((pa) >> 8) & 0xf, ((pa) >> 4) & 0xf, (pa) & 0xf
-
 #define info(fmt, args...) 					\
 	do {							\
 		if (show_info)					\
@@ -184,9 +190,6 @@ int cec_named_ioctl(int fd, const char *name,
 
 #define doioctl(n, r, p) cec_named_ioctl((n)->fd, #r, r, p)
 
-#define cec_phys_addr_exp(pa) \
-        ((pa) >> 12), ((pa) >> 8) & 0xf, ((pa) >> 4) & 0xf, (pa) & 0xf
-
 #define transmit(n, m) (doioctl(n, CEC_TRANSMIT, m))
 
 static inline unsigned ts_to_ms(__u64 ts)
@@ -220,6 +223,7 @@ std::string opcode2s(const struct cec_msg *msg);
 void sad_encode(const struct short_audio_desc *sad, __u32 *descriptor);
 
 // cec-tuner.cpp
+void tuner_dev_info_init(struct state *state);
 void process_tuner_record_timer_msgs(struct node *node, struct cec_msg &msg, unsigned me);
 
 // CEC processing
@@ -227,7 +231,4 @@ void reply_feature_abort(struct node *node, struct cec_msg *msg,
 			 __u8 reason = CEC_OP_ABORT_UNRECOGNIZED_OP);
 void testProcessing(struct node *node, bool wallclock);
 
-// cec-log.c
-void log_msg(const struct cec_msg *msg);
-
 #endif
