@@ -17,9 +17,11 @@
 #include <sys/time.h>
 #include <sys/sysmacros.h>
 #include <dirent.h>
+#include <cstdlib>
 
 #include <linux/media.h>
 
+#include <iostream>
 #include <fstream>
 #include <media-info.h>
 
@@ -199,7 +201,7 @@ std::string mi_get_devpath_from_dev_t(dev_t dev)
 	return devpath;
 }
 
-int mi_get_media_fd(int fd)
+int mi_get_media_fd(int fd, const char *bus_info)
 {
 	int media_fd = -1;
 	dev_t dev;
@@ -220,10 +222,18 @@ int mi_get_media_fd(int fd)
 	media_path[0] = 0;
 	while ((ep = readdir(dp))) {
 		if (!memcmp(ep->d_name, "media", 5) && isdigit(ep->d_name[5])) {
+			struct media_device_info mdinfo;
 			std::string devname("/dev/");
 
 			devname += ep->d_name;
 			media_fd = open(devname.c_str(), O_RDWR);
+
+			if (bus_info &&
+			    (ioctl(media_fd, MEDIA_IOC_DEVICE_INFO, &mdinfo) ||
+			     strcmp(mdinfo.bus_info, bus_info))) {
+				close(media_fd);
+				continue;
+			}
 			break;
 		}
 	}
@@ -419,10 +429,10 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 	media_v2_pad v2_pads[topology.num_pads];
 	media_v2_link v2_links[topology.num_links];
 
-	topology.ptr_entities = (__u64)v2_ents;
-	topology.ptr_interfaces = (__u64)v2_ifaces;
-	topology.ptr_pads = (__u64)v2_pads;
-	topology.ptr_links = (__u64)v2_links;
+	topology.ptr_entities = (uintptr_t)v2_ents;
+	topology.ptr_interfaces = (uintptr_t)v2_ifaces;
+	topology.ptr_pads = (uintptr_t)v2_pads;
+	topology.ptr_links = (uintptr_t)v2_links;
 	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
 		return 0;
 	for (i = 0; i < topology.num_interfaces; i++)
@@ -430,7 +440,7 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 		    v2_ifaces[i].devnode.minor == minor)
 			break;
 	if (i == topology.num_interfaces) {
-		fprintf(stderr, "FAIL: could not find %d:%d device in topology\n",
+		fprintf(stderr, "FAIL: could not find device %d:%d in topology\n",
 			major, minor);
 		if (is_invalid)
 			*is_invalid = true;
@@ -445,30 +455,45 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 		if (v2_links[i].source_id == iface.id)
 			break;
 	}
-	if (i == topology.num_links) {
+	bool is_radio = iface.intf_type == MEDIA_INTF_T_V4L_RADIO;
+
+	if (is_radio && i < topology.num_links) {
 		if (is_invalid)
 			*is_invalid = true;
-		fprintf(stderr, "FAIL: could not find link for interface %u in topology\n",
+		fprintf(stderr, "FAIL: unexpectedly found link for radio interface 0x%08x in topology\n",
 			iface.id);
 		return MEDIA_ENT_F_UNKNOWN;
 	}
-	__u32 ent_id = v2_links[i].sink_id;
-	for (i = 0; i < topology.num_entities; i++) {
-		if (v2_ents[i].id == ent_id)
-			break;
+	if (!is_radio) {
+		if (i == topology.num_links) {
+			if (is_invalid)
+				*is_invalid = true;
+			fprintf(stderr, "FAIL: could not find link for interface 0x%08x in topology\n",
+				iface.id);
+			return MEDIA_ENT_F_UNKNOWN;
+		}
+		__u32 ent_id = v2_links[i].sink_id;
+		for (i = 0; i < topology.num_entities; i++) {
+			if (v2_ents[i].id == ent_id)
+				break;
+		}
+		if (i == topology.num_entities) {
+			if (is_invalid)
+				*is_invalid = true;
+			fprintf(stderr, "FAIL: could not find entity 0x%08x in topology\n",
+				ent_id);
+			return MEDIA_ENT_F_UNKNOWN;
+		}
 	}
-	if (i == topology.num_entities) {
-		if (is_invalid)
-			*is_invalid = true;
-		fprintf(stderr, "FAIL: could not find entity %u in topology\n",
-			ent_id);
-		return MEDIA_ENT_F_UNKNOWN;
-	}
-	media_v2_entity &ent = v2_ents[i];
 
 	printf("Interface Info:\n");
 	printf("\tID               : 0x%08x\n", iface.id);
 	printf("\tType             : %s\n", mi_ifacetype2s(iface.intf_type).c_str());
+
+	if (is_radio)
+		return MEDIA_ENT_F_UNKNOWN;
+
+	media_v2_entity &ent = v2_ents[i];
 	printf("Entity Info:\n");
 	printf("\tID               : 0x%08x (%u)\n", ent.id, ent.id);
 	printf("\tName             : %s\n", ent.name);
@@ -569,7 +594,7 @@ __u32 mi_media_info_for_fd(int media_fd, int fd, bool *is_invalid)
 
 	if (fstat(fd, &sb) == -1) {
 		fprintf(stderr, "failed to stat file\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	ent_id = read_topology(media_fd, major(sb.st_rdev), minor(sb.st_rdev),
