@@ -40,6 +40,7 @@
 	case V4L2_PIX_FMT_Y6: return "6-bit Greyscale";
 	case V4L2_PIX_FMT_Y10: return "10-bit Greyscale";
 	case V4L2_PIX_FMT_Y12: return "12-bit Greyscale";
+	case V4L2_PIX_FMT_Y14: return "14-bit Greyscale";
 	case V4L2_PIX_FMT_Y16: return "16-bit Greyscale";
 	case V4L2_PIX_FMT_Y16_BE: return "16-bit Greyscale BE";
 	case V4L2_PIX_FMT_Y10BPACK: return "10-bit Greyscale (Packed)";
@@ -124,6 +125,10 @@
 	case V4L2_PIX_FMT_SGBRG12P: return "12-bit Bayer GBGB/RGRG Packed";
 	case V4L2_PIX_FMT_SGRBG12P: return "12-bit Bayer GRGR/BGBG Packed";
 	case V4L2_PIX_FMT_SRGGB12P: return "12-bit Bayer RGRG/GBGB Packed";
+	case V4L2_PIX_FMT_SBGGR14: return "14-bit Bayer BGBG/GRGR";
+	case V4L2_PIX_FMT_SGBRG14: return "14-bit Bayer GBGB/RGRG";
+	case V4L2_PIX_FMT_SGRBG14: return "14-bit Bayer GRGR/BGBG";
+	case V4L2_PIX_FMT_SRGGB14: return "14-bit Bayer RGRG/GBGB";
 	case V4L2_PIX_FMT_SBGGR14P: return "14-bit Bayer BGBG/GRGR Packed";
 	case V4L2_PIX_FMT_SGBRG14P: return "14-bit Bayer GBGB/RGRG Packed";
 	case V4L2_PIX_FMT_SGRBG14P: return "14-bit Bayer GRGR/BGBG Packed";
@@ -142,6 +147,23 @@
 	case V4L2_PIX_FMT_KONICA420: return "GSPCA KONICA420";
 	case V4L2_PIX_FMT_HSV24: return "24-bit HSV 8-8-8";
 	case V4L2_PIX_FMT_HSV32: return "32-bit XHSV 8-8-8-8";
+	case V4L2_SDR_FMT_CU8: return "Complex U8";
+	case V4L2_SDR_FMT_CU16LE: return "Complex U16LE";
+	case V4L2_SDR_FMT_CS8: return "Complex S8";
+	case V4L2_SDR_FMT_CS14LE: return "Complex S14LE";
+	case V4L2_SDR_FMT_RU12LE: return "Real U12LE";
+	case V4L2_SDR_FMT_PCU16BE: return "Planar Complex U16BE";
+	case V4L2_SDR_FMT_PCU18BE: return "Planar Complex U18BE";
+	case V4L2_SDR_FMT_PCU20BE: return "Planar Complex U20BE";
+	case V4L2_TCH_FMT_DELTA_TD16: return "16-bit Signed Deltas";
+	case V4L2_TCH_FMT_DELTA_TD08: return "8-bit Signed Deltas";
+	case V4L2_TCH_FMT_TU16: return "16-bit Unsigned Touch Data";
+	case V4L2_TCH_FMT_TU08: return "8-bit Unsigned Touch Data";
+	case V4L2_META_FMT_VSP1_HGO: return "R-Car VSP1 1-D Histogram";
+	case V4L2_META_FMT_VSP1_HGT: return "R-Car VSP1 2-D Histogram";
+	case V4L2_META_FMT_UVC: return "UVC Payload Header Metadata";
+	case V4L2_META_FMT_D4XX: return "Intel D4xx UVC Metadata";
+	case V4L2_META_FMT_VIVID: return "Vivid Metadata";
 	case V4L2_PIX_FMT_MJPEG: return "Motion-JPEG";
 	case V4L2_PIX_FMT_JPEG: return "JFIF JPEG";
 	case V4L2_PIX_FMT_DV: return "1394";
