@@ -221,11 +221,12 @@ CaptureWin::CaptureWin(QScrollArea *sa, QWidget *parent) :
 	m_singleStepNext(false),
 	m_screenTextureCount(0),
 	m_program(0),
-	m_curSize(),
 	m_curIndex(-1),
 	m_nextIndex(-1),
 	m_scrollArea(sa)
 {
+	m_curSize[0] = 0;
+	m_curData[0] = 0;
 	m_canOverrideResolution = false;
 	m_pixelaspect.numerator = 1;
 	m_pixelaspect.denominator = 1;
@@ -369,7 +370,7 @@ void CaptureWin::showCurrentOverrides()
 		printf("%sHSV Encoding: %s\n", prefix, ycbcr_enc2s(m_origHSVEnc).c_str());
 	else if (!m_is_rgb)
 		printf("%sY'CbCr Encoding: %s\n", prefix, ycbcr_enc2s(m_origYCbCrEnc).c_str());
-	printf("%sQuantization Range: %s\n", prefix, quantization2s(m_origQuantization).c_str());
+	printf("%sQuantization: %s\n", prefix, quantization2s(m_origQuantization).c_str());
 	firstTime = false;
 }
 
@@ -571,6 +572,7 @@ void CaptureWin::keyPressEvent(QKeyEvent *event)
 {
 	unsigned w = m_v4l_fmt.g_width();
 	unsigned h = m_v4l_fmt.g_frame_height();
+	unsigned p = m_overrideHorPadding;
 	bool hasShift = event->modifiers() & Qt::ShiftModifier;
 	bool hasCtrl = event->modifiers() & Qt::ControlModifier;
 	bool scalingEnabled = m_canOverrideResolution &&
@@ -587,12 +589,22 @@ void CaptureWin::keyPressEvent(QKeyEvent *event)
 		if (!m_scrollArea->isFullScreen())
 			return;
 	case Qt::Key_Left:
-		if (scalingEnabled && w > 16)
-			w -= 2;
+		if (hasShift) {
+			if (scalingEnabled && p >= 2)
+				p -= 2;
+		} else {
+			if (scalingEnabled && w > 16)
+				w -= 2;
+		}
 		break;
 	case Qt::Key_Right:
-		if (scalingEnabled && w < 10240)
-			w += 2;
+		if (hasShift) {
+			if (scalingEnabled && p < 10240)
+				p += 2;
+		} else {
+			if (scalingEnabled && w < 10240)
+				w += 2;
+		}
 		break;
 	case Qt::Key_Up:
 		if (scalingEnabled && h > 16)
@@ -675,6 +687,12 @@ void CaptureWin::keyPressEvent(QKeyEvent *event)
 			m_scrollArea->resize(w, h);
 		else
 			resize(w, h);
+
+		if ((w + p) != m_v4l_fmt.g_bytesperline()) {
+			printf("New horizontal resolution: %u + %u (%u)\n", w, p, w + p);
+			setOverrideHorPadding(p);
+			updateShader();
+		}
 	}
 }
 
@@ -719,7 +737,7 @@ void CaptureWin::checkError(const char *msg)
 		fprintf(stderr, "OpenGL Error (no: %u) code 0x%x: %s.\n", errNo++, err, msg);
 
 	if (errNo)
-		exit(errNo);
+		std::exit(errNo);
 }
 
 void CaptureWin::configureTexture(size_t idx)
@@ -747,6 +765,14 @@ void CaptureWin::setOverrideHeight(__u32 h)
 		m_resolutionOverride->setChecked(true);
 }
 
+void CaptureWin::setOverrideHorPadding(__u32 p)
+{
+	m_overrideHorPadding = p;
+
+	if (!m_overrideHorPadding && m_canOverrideResolution)
+		m_resolutionOverride->setChecked(true);
+}
+
 void CaptureWin::setModeV4L2(cv4l_fd *fd)
 {
 	m_mode = AppModeV4L2;
@@ -791,7 +817,7 @@ void CaptureWin::setModeFile(const QString &filename)
 	m_file.setFileName(filename);
 	if (!m_file.open(QIODevice::ReadOnly)) {
 		fprintf(stderr, "could not open %s\n", filename.toUtf8().data());
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	m_canOverrideResolution = true;
 }
@@ -953,6 +979,8 @@ bool CaptureWin::setV4LFormat(cv4l_fmt &fmt)
 	}
 	if (m_mode == AppModeFile && m_overrideWidth)
 		fmt.s_width(m_overrideWidth);
+	if (m_mode == AppModeFile && m_overrideHorPadding)
+		fmt.s_bytesperline(fmt.g_bytesperline() + m_overrideHorPadding);
 	if (m_mode == AppModeFile && m_overrideField != 0xffffffff)
 		fmt.s_field(m_overrideField);
 	if (m_mode == AppModeFile && m_overrideHeight)
@@ -1062,7 +1090,7 @@ bool CaptureWin::setV4LFormat(cv4l_fmt &fmt)
 		}
 		printf("Width x Height:    %ux%u\n", m_v4l_fmt.g_width(), m_v4l_fmt.g_height());
 		printf("Field:             %s\n", field2s(m_v4l_fmt.g_field()).c_str());
-		printf("Pixelformat:       %s ('%s')\n", fmt.description, fcc2s(m_v4l_fmt.g_pixelformat()).c_str());
+		printf("Pixel Format:      %s ('%s')\n", fmt.description, fcc2s(m_v4l_fmt.g_pixelformat()).c_str());
 		printf("Colorspace:        %s\n", colorspace2s(m_v4l_fmt.g_colorspace()).c_str());
 		if (m_is_hsv)
 			printf("HSV Encoding:      %s\n", ycbcr_enc2s(m_v4l_fmt.g_hsv_enc()).c_str());
@@ -1109,7 +1137,7 @@ void CaptureWin::v4l2ReadEvent()
 	if (m_cnt == 0)
 		return;
 	if (--m_cnt == 0)
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 }
 
 void CaptureWin::v4l2ExceptionEvent()
@@ -1125,7 +1153,7 @@ void CaptureWin::v4l2ExceptionEvent()
 				fprintf(stderr, "Unsupported format: '%s' %s\n",
 					fcc2s(fmt.g_pixelformat()).c_str(),
 					pixfmt2s(fmt.g_pixelformat()).c_str());
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			updateOrigValues();
 			m_updateShader = true;
@@ -1292,7 +1320,7 @@ void CaptureWin::sockReadEvent()
 	if (m_cnt == 0)
 		return;
 	if (--m_cnt == 0)
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 	return;
 
 new_conn:
@@ -1350,6 +1378,9 @@ void CaptureWin::initImageFormat()
 	tpg_s_quantization(&m_tpg, m_v4l_fmt.g_quantization());
 	m_v4l_fmt.s_num_planes(tpg_g_buffers(&m_tpg));
 	for (unsigned p = 0; p < m_v4l_fmt.g_num_planes(); p++) {
+		if (m_mode == AppModeFile && m_overrideHorPadding)
+			tpg_s_bytesperline(&m_tpg, p, tpg_g_bytesperline(&m_tpg, p) + m_overrideHorPadding);
+
 		m_v4l_fmt.s_bytesperline(tpg_g_bytesperline(&m_tpg, p), p);
 		m_v4l_fmt.s_sizeimage(tpg_calc_plane_size(&m_tpg, p), p);
 	}
@@ -1464,7 +1495,7 @@ void CaptureWin::tpgUpdateFrame()
 	delete m_timer;
 	m_timer = NULL;
 	if (!m_test)
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 
 	m_cnt = m_test;
 
@@ -1502,7 +1533,7 @@ void CaptureWin::tpgUpdateFrame()
 								m_testState.field_idx = 0;
 							if (mask_fmt ||
 							    formats[++m_testState.fmt_idx] == 0)
-								exit(0);
+								std::exit(EXIT_SUCCESS);
 						}
 					}
 				}
@@ -1512,7 +1543,7 @@ void CaptureWin::tpgUpdateFrame()
 
 	while (!supportedFmt(formats[m_testState.fmt_idx]))
 		if (formats[++m_testState.fmt_idx] == 0)
-			exit(0);
+			std::exit(EXIT_SUCCESS);
 
 	m_v4l_fmt.s_pixelformat(formats[m_testState.fmt_idx]);
 	updateV4LFormat(m_v4l_fmt);
