@@ -28,6 +28,7 @@
 #include <libv4l2rds.h>
 
 #include <cctype>
+#include <cmath>
 #include <ctime>
 #include <list>
 #include <vector>
@@ -107,12 +108,12 @@ static struct option long_options[] = {
 	{0, 0, 0, 0}
 };
 
-static void usage_hint(void)
+static void usage_hint()
 {
 	fprintf(stderr, "Try 'rds-ctl --help' for more information.\n");
 }
 
-static void usage_common(void)
+static void usage_common()
 {
 	printf("\nGeneral/Common options:\n"
 	       "  --all              display all device information available\n"
@@ -126,7 +127,7 @@ static void usage_common(void)
 	       );
 }
 
-static void usage_tuner(void)
+static void usage_tuner()
 {
 	printf("\nTuner/Modulator options:\n"
 	       "  -F, --get-freq     query the frequency [VIDIOC_G_FREQUENCY]\n"
@@ -144,7 +145,7 @@ static void usage_tuner(void)
 	       );
 }
 
-static void usage_rds(void)
+static void usage_rds()
 {
 	printf("\nRDS options: \n"
 	       "  -b, --rbds         parse the RDS data according to the RBDS standard\n"
@@ -163,7 +164,7 @@ static void usage_rds(void)
 	       );
 }
 
-static void usage(void)
+static void usage()
 {
 	printf("Usage:\n");
 	usage_common();
@@ -313,9 +314,9 @@ static void print_devices(dev_vec files)
 			continue;
 		doioctl(fd, VIDIOC_QUERYCAP, &vcap);
 		close(fd);
-		bus_info = (const char *)vcap.bus_info;
+		bus_info = reinterpret_cast<const char *>(vcap.bus_info);
 	if (cards[bus_info].empty())
-			cards[bus_info] += std::string((char *)vcap.card)
+			cards[bus_info] += std::string(reinterpret_cast<char *>(vcap.card))
 				+ " (" + bus_info + "):\n";
 		cards[bus_info] += "\t" + (*iter);
 		cards[bus_info] += "\n";
@@ -325,7 +326,7 @@ static void print_devices(dev_vec files)
 		printf("%s\n", iter->second.c_str());
 	}
 }
-static dev_vec list_devices(void)
+static dev_vec list_devices()
 {
 	DIR *dp;
 	struct dirent *ep;
@@ -338,7 +339,7 @@ static dev_vec list_devices(void)
 	dp = opendir("/dev");
 	if (dp == NULL) {
 		perror ("Couldn't open the directory");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	while ((ep = readdir(dp)))
 		if (is_radio_dev(ep->d_name))
@@ -374,7 +375,7 @@ static dev_vec list_devices(void)
 
 static int parse_subopt(char **subs, const char * const *subopts, char **value)
 {
-	int opt = getsubopt(subs, (char * const *)subopts, value);
+	int opt = getsubopt(subs, const_cast<char * const *>(subopts), value);
 
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
@@ -413,7 +414,7 @@ static void parse_freq_seek(char *optarg, struct v4l2_hw_freq_seek &seek)
 			break;
 		default:
 			usage_tuner();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 }
@@ -520,20 +521,20 @@ static void print_rds_statistics(const struct v4l2_rds_statistics *statistics)
 	printf("received blocks / received groups: %u / %u\n",
 		statistics->block_cnt, statistics->group_cnt);
 
-	float block_error_percentage = 0;
+	double block_error_percentage = 0;
 
 	if (statistics->block_cnt)
 		block_error_percentage =
-			((float)statistics->block_error_cnt / statistics->block_cnt) * 100.0;
+			(static_cast<float>(statistics->block_error_cnt) / statistics->block_cnt) * 100.0;
 	printf("block errors / group errors: %u (%3.2f%%) / %u \n",
 		statistics->block_error_cnt,
 		block_error_percentage, statistics->group_error_cnt);
 
-	float block_corrected_percentage = 0;
+	double block_corrected_percentage = 0;
 
 	if (statistics->block_cnt)
 		block_corrected_percentage = (
-			(float)statistics->block_corrected_cnt / statistics->block_cnt) * 100.0;
+			static_cast<float>(statistics->block_corrected_cnt) / statistics->block_cnt) * 100.0;
 	printf("corrected blocks: %u (%3.2f%%)\n",
 		statistics->block_corrected_cnt, block_corrected_percentage);
 	for (int i = 0; i < 16; i++)
@@ -691,7 +692,8 @@ static void read_rds(struct v4l2_rds *handle, const int fd, const int wait_limit
 			if (byte_cnt == 0) {
 				printf("\nEnd of input file reached \n");
 				break;
-			} else if(++error_cnt > 2) {
+			}
+			if (++error_cnt > 2) {
 				fprintf(stderr, "\nError reading from "
 					"device (no RDS data available)\n");
 				break;
@@ -722,7 +724,7 @@ static void read_rds_from_fd(const int fd)
 	/* create an rds handle for the current device */
 	if (!(rds_handle = v4l2_rds_create(params.options[OptRBDS]))) {
 		fprintf(stderr, "Failed to init RDS lib: %s\n", strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	/* try to receive and decode RDS data */
@@ -746,7 +748,7 @@ static int parse_cl(int argc, char **argv)
 
 	if (argc == 1) {
 		usage_hint();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	for (i = 0; long_options[i].name; i++) {
 		if (!isalpha(long_options[i].val))
@@ -755,7 +757,7 @@ static int parse_cl(int argc, char **argv)
 		if (long_options[i].has_arg == required_argument)
 			short_options[idx++] = ':';
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -764,7 +766,7 @@ static int parse_cl(int argc, char **argv)
 		if (opt == -1)
 			break;
 
-		params.options[(int)opt] = 1;
+		params.options[opt] = 1;
 		switch (opt) {
 		case OptSetDevice:
 			strncpy(params.fd_name, optarg, sizeof(params.fd_name) - 1);
@@ -915,8 +917,8 @@ static void get_options(const int fd, const int capabilities, struct v4l2_freque
 			else
 				printf("\tFrequency range      : %.1f MHz - %.1f MHz\n",
 					 vt.rangelow / 16.0, vt.rangehigh / 16.0);
-			printf("\tSignal strength/AFC  : %d%%/%d\n",
-				(int)((vt.signal / 655.35)+0.5), vt.afc);
+			printf("\tSignal strength/AFC  : %ld%%/%d\n",
+				lround(vt.signal / 655.25), vt.afc);
 			printf("\tCurrent audio mode   : %s\n", audmode2s(vt.audmode));
 			printf("\tAvailable subchannels: %s\n",
 					rxsubchans2s(vt.rxsubchans).c_str());
@@ -974,27 +976,27 @@ int main(int argc, char **argv)
 	parse_cl(argc, argv);
 	if (params.options[OptHelp]) {
 		usage();
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 	}
 
 	/* File Mode: disables all other features, except for RDS decoding */
 	if (params.filemode_active) {
 		if ((fd = open(params.fd_name, O_RDONLY|O_NONBLOCK)) < 0){
 			perror("error opening file");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		read_rds_from_fd(fd);
 		close(fd);
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 	}
 
 	/* Device Mode: open the radio device as read-only and non-blocking */
 	if (!params.options[OptSetDevice]) {
 		/* check the system for RDS capable devices */
 		dev_vec devices = list_devices();
-		if (devices.size() == 0) {
+		if (devices.empty()) {
 			fprintf(stderr, "No RDS-capable device found\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		strncpy(params.fd_name, devices[0].c_str(), sizeof(params.fd_name));
 		params.fd_name[sizeof(params.fd_name) - 1] = '\0';
@@ -1003,7 +1005,7 @@ int main(int argc, char **argv)
 	if ((fd = open(params.fd_name, O_RDONLY | O_NONBLOCK)) < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", params.fd_name,
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	doioctl(fd, VIDIOC_QUERYCAP, &vcap);
 
@@ -1019,5 +1021,5 @@ int main(int argc, char **argv)
 		read_rds_from_fd(fd);
 
 	close(fd);
-	exit(app_result);
+	std::exit(app_result);
 }
