@@ -18,10 +18,12 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
+#include <cstdlib>
+#include <cstring>
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 #include <inttypes.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -155,7 +157,7 @@ static struct option long_options[] = {
 	{0, 0, 0, 0}
 };
 
-static void usage(void)
+static void usage()
 {
 	printf("Usage:\n");
 	printf("Common options:\n");
@@ -247,7 +249,6 @@ static void usage(void)
 	printf("  -w, --wrapper      Use the libv4l2 wrapper library.\n");
 #endif
 	printf("  -W, --exit-on-warn Exit on the first warning.\n");
-	exit(0);
 }
 
 const char *ok(int res)
@@ -285,12 +286,12 @@ int check_string(const char *s, size_t len)
 
 int check_ustring(const __u8 *s, int len)
 {
-	return check_string((const char *)s, len);
+	return check_string(reinterpret_cast<const char *>(s), len);
 }
 
 int check_0(const void *p, int len)
 {
-	const __u8 *q = (const __u8 *)p;
+	const __u8 *q = static_cast<const __u8 *>(p);
 
 	while (len--)
 		if (*q++)
@@ -301,7 +302,7 @@ int check_0(const void *p, int len)
 static std::map<std::string, std::string> stream_from_map;
 static std::map<std::string, bool> stream_hdr_map;
 
-std::string stream_from(std::string pixelformat, bool &use_hdr)
+std::string stream_from(const std::string &pixelformat, bool &use_hdr)
 {
 	if (stream_from_map.find(pixelformat) == stream_from_map.end()) {
 		if (pixelformat.empty())
@@ -428,7 +429,8 @@ static void restoreState()
 	if (node->has_outputs) {
 		node->s_output(state.output.index);
 		if (state.output.audioset)
-		node->s_audout(state.aoutput.index);
+			node->s_audout(state.aoutput.index);
+
 		if (node->g_caps() & V4L2_CAP_MODULATOR) {
 			node->s_modulator(state.modulator);
 			node->s_frequency(state.freq);
@@ -481,11 +483,12 @@ static void restoreState()
 	node->reopen();
 }
 
+__attribute__((noreturn))
 static void signal_handler_interrupt(int signum)
 {
 	restoreState();
 	printf("\n");
-	exit(-1);
+	std::exit(EXIT_FAILURE);
 }
 
 static void determine_codec_mask(struct node &node)
@@ -613,6 +616,7 @@ static int testCap(struct node *node)
 
 	memset(&vcap, 0xff, sizeof(vcap));
 	// Must always be there
+	fail_on_test(doioctl(node, VIDIOC_QUERYCAP, NULL) != EFAULT);
 	fail_on_test(doioctl(node, VIDIOC_QUERYCAP, &vcap));
 	fail_on_test(check_ustring(vcap.driver, sizeof(vcap.driver)));
 	fail_on_test(check_ustring(vcap.card, sizeof(vcap.card)));
@@ -628,7 +632,7 @@ static int testCap(struct node *node)
 	    memcmp(vcap.bus_info, "rmi4:", 5))
 		return fail("missing bus_info prefix ('%s')\n", vcap.bus_info);
 	if (!node->media_bus_info.empty() &&
-	    node->media_bus_info != std::string((const char *)vcap.bus_info))
+	    node->media_bus_info != std::string(reinterpret_cast<const char *>(vcap.bus_info)))
 		warn("media bus_info '%s' differs from V4L2 bus_info '%s'\n",
 		     node->media_bus_info.c_str(), vcap.bus_info);
 	fail_on_test((vcap.version >> 16) < 3);
@@ -763,6 +767,26 @@ static int testPrio(struct node *node, struct node *node2)
 	return 0;
 }
 
+static int testInvalidIoctls(struct node *node, char type)
+{
+	unsigned ioc = _IOC(_IOC_NONE, type, 0xff, 0);
+	unsigned char buf[0x4000] = {};
+
+	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	ioc = _IOC(_IOC_NONE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	ioc = _IOC(_IOC_READ, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	ioc = _IOC(_IOC_WRITE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	ioc = _IOC(_IOC_READ | _IOC_WRITE, type, 0, 0x3fff);
+	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
+	fail_on_test(check_0(buf, sizeof(buf)));
+	return 0;
+}
+
 static void streamingSetup(struct node *node)
 {
 	if (node->can_capture) {
@@ -786,7 +810,7 @@ static void streamingSetup(struct node *node)
 
 static int parse_subopt(char **subs, const char * const *subopts, char **value)
 {
-	int opt = getsubopt(subs, (char * const *)subopts, value);
+	int opt = getsubopt(subs, const_cast<char * const *>(subopts), value);
 
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
@@ -849,7 +873,7 @@ static std::string make_devname(const char *device, const char *devname,
 		return device;
 	if (is_media) {
 		close(media_fd);
-		return media_devname.c_str();
+		return media_devname;
 	}
 
 	media_v2_topology topology;
@@ -860,11 +884,11 @@ static std::string make_devname(const char *device, const char *devname,
 	}
 
 	media_v2_entity *ents = new media_v2_entity[topology.num_entities];
-	topology.ptr_entities = (__u64)ents;
+	topology.ptr_entities = (uintptr_t)ents;
 	media_v2_link *links = new media_v2_link[topology.num_links];
-	topology.ptr_links = (__u64)links;
+	topology.ptr_links = (uintptr_t)links;
 	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
-	topology.ptr_interfaces = (__u64)ifaces;
+	topology.ptr_interfaces = (uintptr_t)ifaces;
 
 	unsigned i, ent_id, iface_id = 0;
 
@@ -899,7 +923,7 @@ static std::string make_devname(const char *device, const char *devname,
 		goto err;
 
 	result = mi_media_get_device(ifaces[i].devnode.major, ifaces[i].devnode.minor);
-	
+
 err:
 	delete [] ents;
 	delete [] links;
@@ -925,10 +949,10 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 
 	if (node.is_v4l2()) {
 		doioctl(&node, VIDIOC_QUERYCAP, &vcap);
-		driver = (const char *)vcap.driver;
+		driver = reinterpret_cast<const char *>(vcap.driver);
 		is_vivid = driver == "vivid";
 		if (is_vivid)
-			node.bus_info = (const char *)vcap.bus_info;
+			node.bus_info = reinterpret_cast<const char *>(vcap.bus_info);
 		determine_codec_mask(node);
 	} else {
 		memset(&vcap, 0, sizeof(vcap));
@@ -956,10 +980,12 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		       driver.c_str(), node.device, node.g_direct() ? "" : " (using libv4l2)");
 
 	if (node.g_caps() & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VBI_CAPTURE |
-			 V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_SLICED_VBI_CAPTURE))
+			 V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_SLICED_VBI_CAPTURE |
+			 V4L2_CAP_META_CAPTURE))
 		node.has_inputs = true;
 	if (node.g_caps() & (V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VBI_OUTPUT |
-			 V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_SLICED_VBI_OUTPUT))
+			 V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_SLICED_VBI_OUTPUT |
+			 V4L2_CAP_META_OUTPUT))
 		node.has_outputs = true;
 	if (node.g_caps() & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VBI_CAPTURE |
 			 V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE |
@@ -980,6 +1006,8 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		node.is_video = false;
 		node.is_meta = true;
 	}
+	if (node.g_caps() & V4L2_CAP_IO_MC)
+		node.is_io_mc = true;
 
 	/* Information Opts */
 
@@ -1016,14 +1044,14 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	else if (media_fd >= 0)
 		ent_id = mi_media_info_for_fd(media_fd, node.g_fd(), &is_invalid);
 
-	if (ent_id && ent_id != MEDIA_ENT_F_UNKNOWN) {
+	if (ent_id != MEDIA_ENT_F_UNKNOWN) {
 		memset(&node.entity, 0, sizeof(node.entity));
 		node.entity.id = ent_id;
 		if (!ioctl(media_fd, MEDIA_IOC_ENUM_ENTITIES, &node.entity)) {
 			struct media_links_enum links_enum;
 
 			node.pads = new media_pad_desc[node.entity.pads];
-			node.links = new media_link_desc[node.entity.links];	
+			node.links = new media_link_desc[node.entity.links];
 			memset(&links_enum, 0, sizeof(links_enum));
 			links_enum.entity = ent_id;
 			links_enum.pads = node.pads;
@@ -1105,10 +1133,13 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 			       node.topology->num_links);
 		printf("\ttest MEDIA_IOC_ENUM_ENTITIES/LINKS: %s\n", ok(testMediaEnum(&node)));
 		printf("\ttest MEDIA_IOC_SETUP_LINK: %s\n", ok(testMediaSetupLink(&node)));
+		printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, '|')));
 		printf("\n");
 		goto show_total;
 	}
 
+	printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, 'V')));
+
 	/* Debug ioctls */
 
 	printf("Debug ioctls:\n");
@@ -1456,11 +1487,12 @@ int main(int argc, char **argv)
 #ifdef SHA
 #define STR(x) #x
 #define STRING(x) STR(x)
-	printf("v4l2-compliance SHA: %s, %zd bits\n", STRING(SHA), sizeof(void *) * 8);
+	printf("v4l2-compliance SHA: %s", STRING(SHA));
 #else
-	printf("v4l2-compliance SHA: not available, %zd bits\n", sizeof(void *) * 8);
+	printf("v4l2-compliance SHA: not available\n");
 #endif
 
+	printf(", %zd bits, %zd-bit time_t\n", sizeof(void *) * 8, sizeof(time_t) * 8);
 	uname(&uts);
 	sscanf(uts.release, "%d.%d.%d", &v1, &v2, &v3);
 	if (v1 == 2 && v2 == 6)
@@ -1491,7 +1523,7 @@ int main(int argc, char **argv)
 			short_options[idx++] = ':';
 		}
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -1500,7 +1532,7 @@ int main(int argc, char **argv)
 		if (ch == -1)
 			break;
 
-		options[(int)ch] = 1;
+		options[ch] = 1;
 		if (!option_index) {
 			for (i = 0; long_options[i].val; i++) {
 				if (long_options[i].val == ch) {
@@ -1516,7 +1548,7 @@ int main(int argc, char **argv)
 		switch (ch) {
 		case OptHelp:
 			usage();
-			return 0;
+			std::exit(EXIT_FAILURE);
 		case OptSetDevice:
 			device = make_devname(optarg, "video", media_bus_info);
 			break;
@@ -1552,7 +1584,7 @@ int main(int argc, char **argv)
 			break;
 		case OptStreamFrom:
 		case OptStreamFromHdr: {
-			char *equal = strchr(optarg, '=');
+			char *equal = std::strchr(optarg, '=');
 			bool has_hdr = ch == OptStreamFromHdr;
 
 			if (equal == optarg)
@@ -1591,7 +1623,7 @@ int main(int argc, char **argv)
 						color_component = 2;
 					else {
 						usage();
-						exit(1);
+						std::exit(EXIT_FAILURE);
 					}
 					break;
 				case 1:
@@ -1606,7 +1638,7 @@ int main(int argc, char **argv)
 					break;
 				default:
 					usage();
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 			}
 			break;
@@ -1619,7 +1651,7 @@ int main(int argc, char **argv)
 				show_colors = isatty(STDOUT_FILENO);
 			else {
 				usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			break;
 		case OptNoWarnings:
@@ -1641,13 +1673,13 @@ int main(int argc, char **argv)
 			fprintf(stderr, "Option `%s' requires a value\n",
 				argv[optind]);
 			usage();
-			return 1;
+			std::exit(EXIT_FAILURE);
 		case '?':
 			if (argv[optind])
 				fprintf(stderr, "Unknown argument `%s'\n",
 					argv[optind]);
 			usage();
-			return 1;
+			std::exit(EXIT_FAILURE);
 		}
 	}
 	if (optind < argc) {
@@ -1656,7 +1688,7 @@ int main(int argc, char **argv)
 			fprintf(stderr, "%s ", argv[optind++]);
 		fprintf(stderr, "\n");
 		usage();
-		return 1;
+		std::exit(EXIT_FAILURE);
 	}
 	bool direct = !options[OptUseWrapper];
 	int fd;
@@ -1666,12 +1698,12 @@ int main(int argc, char **argv)
 	if (type == MEDIA_TYPE_CANT_STAT) {
 		fprintf(stderr, "Cannot open device %s, exiting.\n",
 			device.c_str());
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	if (type == MEDIA_TYPE_UNKNOWN) {
 		fprintf(stderr, "Unable to detect what device %s is, exiting.\n",
 			device.c_str());
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	node.device = device.c_str();
@@ -1693,7 +1725,7 @@ int main(int argc, char **argv)
 	if (fd < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	if (!expbuf_device.empty()) {
@@ -1703,7 +1735,7 @@ int main(int argc, char **argv)
 		if (fd < 0) {
 			fprintf(stderr, "Failed to open %s: %s\n", expbuf_device.c_str(),
 				strerror(errno));
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 
@@ -1714,5 +1746,5 @@ int main(int argc, char **argv)
 	if (media_fd >= 0)
 		close(media_fd);
 
-	exit(app_result);
+	std::exit(app_result);
 }
