@@ -48,10 +48,12 @@
 #define VIVID_CID_QUEUE_ERROR		(VIVID_CID_VIVID_BASE + 70)
 #define VIVID_CID_REQ_VALIDATE_ERROR	(VIVID_CID_VIVID_BASE + 72)
 
-static struct cv4l_fmt cur_fmt;
-static struct cv4l_fmt cur_m2m_fmt;
+static cv4l_fmt cur_fmt;
+static cv4l_fmt cur_m2m_fmt;
 static int stream_from_fd = -1;
 static bool stream_use_hdr;
+static unsigned max_bytesused[VIDEO_MAX_PLANES];
+static unsigned min_data_offset[VIDEO_MAX_PLANES];
 
 bool operator<(struct timeval const& n1, struct timeval const& n2)
 {
@@ -159,7 +161,7 @@ static bool fill_output_buffer(const cv4l_queue &q, cv4l_buffer &buf, bool first
 		if (!stream_use_hdr) {
 			ssize_t sz = read(stream_from_fd, q.g_dataptr(buf.g_index(), p), len);
 
-			if (sz < len) {
+			if (sz < static_cast<ssize_t>(len)) {
 				seek = true;
 				break;
 			}
@@ -181,7 +183,7 @@ static bool fill_output_buffer(const cv4l_queue &q, cv4l_buffer &buf, bool first
 
 		ssize_t sz = read(stream_from_fd, q.g_dataptr(buf.g_index(), p), bytesused);
 
-		if (sz < bytesused) {
+		if (sz < static_cast<ssize_t>(bytesused)) {
 			seek = true;
 			break;
 		}
@@ -354,7 +356,7 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 	unsigned timestamp_src = g_timestamp_src();
 	unsigned frame_types = 0;
 	unsigned buf_states = 0;
-	const struct cv4l_fmt &fmt = is_m2m ? cur_m2m_fmt : cur_fmt;
+	const cv4l_fmt &fmt = is_m2m ? cur_m2m_fmt : cur_fmt;
 
 	fail_on_test(g_type() != type);
 	fail_on_test(g_memory() == 0);
@@ -430,26 +432,26 @@ int buffer::check(unsigned type, unsigned memory, unsigned index,
 				fail_on_test(g_field() == seq.last_field);
 				seq.field_nr ^= 1;
 				if (seq.field_nr) {
-					if ((int)g_sequence() != seq.last_seq)
+					if (static_cast<int>(g_sequence()) != seq.last_seq)
 						warn("got sequence number %u, expected %u\n",
 							g_sequence(), seq.last_seq);
 				} else {
 					fail_on_test((int)g_sequence() == seq.last_seq + 1);
-					if ((int)g_sequence() != seq.last_seq + 1)
+					if (static_cast<int>(g_sequence()) != seq.last_seq + 1)
 						warn("got sequence number %u, expected %u\n",
 							g_sequence(), seq.last_seq + 1);
 				}
 			} else {
 				fail_on_test(g_field() != fmt.g_field());
-				if ((int)g_sequence() != seq.last_seq + 1)
+				if (static_cast<int>(g_sequence()) != seq.last_seq + 1)
 					warn("got sequence number %u, expected %u\n",
 							g_sequence(), seq.last_seq + 1);
 			}
-		} else if ((int)g_sequence() != seq.last_seq + 1) {
+		} else if (static_cast<int>(g_sequence()) != seq.last_seq + 1) {
 			warn("got sequence number %u, expected %u\n",
 					g_sequence(), seq.last_seq + 1);
 		}
-		seq.last_seq = (int)g_sequence();
+		seq.last_seq = static_cast<int>(g_sequence());
 		seq.last_field = g_field();
 	} else {
 		fail_on_test(g_sequence());
@@ -543,7 +545,7 @@ int testReqBufs(struct node *node)
 	bool dmabuf_valid;
 	int ret;
 	unsigned i, m;
-	
+
 	node->reopen();
 
 	cv4l_queue q(0, 0);
@@ -768,26 +770,29 @@ int testReadWrite(struct node *node)
 	bool can_rw = node->g_caps() & V4L2_CAP_READWRITE;
 	int fd_flags = fcntl(node->g_fd(), F_GETFL);
 	char buf = 0;
-	int ret;
-
-	if (v4l_has_vbi(node->g_v4l_fd()) &&
-	    !(node->cur_io_caps & V4L2_IN_CAP_STD)) {
-		return 0;
-	}
+	int ret, ret2;
+	int err, err2;
 
 	fcntl(node->g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
+	errno = 0;
 	if (node->can_capture)
 		ret = node->read(&buf, 1);
 	else
 		ret = node->write(&buf, 1);
+	err = errno;
+	fail_on_test(v4l_has_vbi(node->g_v4l_fd()) &&
+		     !(node->cur_io_caps & V4L2_IN_CAP_STD) && ret >= 0);
+
 	// Note: RDS can only return multiples of 3, so we accept
 	// both 0 and 1 as return code.
 	// EBUSY can be returned when attempting to read/write to a
 	// multiplanar format.
+	// EINVAL can be returned if read()/write() is not supported
+	// for the current input/output.
 	if (can_rw)
-		fail_on_test((ret < 0 && errno != EAGAIN && errno != EBUSY) || ret > 1);
+		fail_on_test((ret < 0 && err != EAGAIN && err != EBUSY && err != EINVAL) || ret > 1);
 	else
-		fail_on_test(ret >= 0 || errno != EINVAL);
+		fail_on_test(ret >= 0 || err != EINVAL);
 	if (!can_rw)
 		return ENOTTY;
 
@@ -795,11 +800,13 @@ int testReadWrite(struct node *node)
 	fcntl(node->g_fd(), F_SETFL, fd_flags | O_NONBLOCK);
 
 	/* check that the close cleared the busy flag */
+	errno = 0;
 	if (node->can_capture)
-		ret = node->read(&buf, 1);
+		ret2 = node->read(&buf, 1);
 	else
-		ret = node->write(&buf, 1);
-	fail_on_test((ret < 0 && errno != EAGAIN && errno != EBUSY) || ret > 1);
+		ret2 = node->write(&buf, 1);
+	err2 = errno;
+	fail_on_test(ret2 != ret || err2 != err);
 	return 0;
 }
 
@@ -973,12 +980,19 @@ static int captureBufs(struct node *node, struct node *node_m2m_cap, const cv4l_
 		if (ret != EAGAIN) {
 			fail_on_test(ret);
 			if (show_info)
-				printf("\t\t%s Buffer: %d Sequence: %d Field: %s Size: %d Flags: %s Timestamp: %ld.%06lds\n",
+				printf("\t\t%s Buffer: %d Sequence: %d Field: %s Size: %d Flags: %s Timestamp: %lld.%06llds\n",
 				       v4l_type_is_output(buf.g_type()) ? "Out" : "Cap",
 				       buf.g_index(), buf.g_sequence(),
 				       field2s(buf.g_field()).c_str(), buf.g_bytesused(),
 				       bufferflags2s(buf.g_flags()).c_str(),
-				       buf.g_timestamp().tv_sec, buf.g_timestamp().tv_usec);
+				       static_cast<__u64>(buf.g_timestamp().tv_sec),  static_cast<__u64>(buf.g_timestamp().tv_usec));
+			for (unsigned p = 0; p < buf.g_num_planes(); p++) {
+				if (max_bytesused[p] < buf.g_bytesused(p))
+					max_bytesused[p] = buf.g_bytesused(p);
+				if (min_data_offset[p] > buf.g_data_offset(p))
+					min_data_offset[p] = buf.g_data_offset(p);
+			}
+
 			fail_on_test(buf.check(q, last_seq));
 			if (!show_info && !no_progress) {
 				printf("\r\t%s: Frame #%03d%s",
@@ -1010,7 +1024,10 @@ static int captureBufs(struct node *node, struct node *node_m2m_cap, const cv4l_
 				buf.s_request_fd(buf_req_fds[req_idx]);
 			}
 			fail_on_test(buf.qbuf(node, q));
-			fail_on_test(buf.g_flags() & V4L2_BUF_FLAG_DONE);
+			// This is not necessarily wrong (v4l-touch drivers can do this),
+			// but it is certainly unusual enough to warn about this.
+			if (buf.g_flags() & V4L2_BUF_FLAG_DONE)
+				warn_once("QBUF returned the buffer as DONE.\n");
 			if (buf.g_flags() & V4L2_BUF_FLAG_REQUEST_FD) {
 				fail_on_test(doioctl_fd(buf_req_fds[req_idx],
 							MEDIA_REQUEST_IOC_QUEUE, 0));
@@ -1050,12 +1067,12 @@ static int captureBufs(struct node *node, struct node *node_m2m_cap, const cv4l_
 		capture_count++;
 
 		if (show_info)
-			printf("\t\t%s Buffer: %d Sequence: %d Field: %s Size: %d Flags: %s Timestamp: %ld.%06lds\n",
+			printf("\t\t%s Buffer: %d Sequence: %d Field: %s Size: %d Flags: %s Timestamp: %lld.%06llds\n",
 			       v4l_type_is_output(buf.g_type()) ? "Out" : "Cap",
 			       buf.g_index(), buf.g_sequence(),
 			       field2s(buf.g_field()).c_str(), buf.g_bytesused(),
 			       bufferflags2s(buf.g_flags()).c_str(),
-			       buf.g_timestamp().tv_sec, buf.g_timestamp().tv_usec);
+			       static_cast<__u64>(buf.g_timestamp().tv_sec), static_cast<__u64>(buf.g_timestamp().tv_usec));
 		fail_on_test(ret);
 		if (v4l_type_is_capture(buf.g_type()) && buf.g_bytesused())
 			fail_on_test(buf.check(m2m_q, last_m2m_seq, true));
@@ -1110,7 +1127,7 @@ static int captureBufs(struct node *node, struct node *node_m2m_cap, const cv4l_
 	}
 	if (node->is_m2m)
 		printf("\t%s: Captured %d buffers\n", buftype2s(m2m_q.g_type()).c_str(), capture_count);
-	
+
 	return 0;
 }
 
@@ -1232,7 +1249,7 @@ int testMmap(struct node *node, struct node *node_m2m_cap, unsigned frame_count,
 
 		cv4l_queue q(type, V4L2_MEMORY_MMAP);
 		cv4l_queue m2m_q(v4l_type_invert(type));
-	
+
 		if (testSetupVbi(node, type))
 			continue;
 
@@ -1456,8 +1473,6 @@ int testMmap(struct node *node, struct node *node_m2m_cap, unsigned frame_count,
 
 static int setupUserPtr(struct node *node, cv4l_queue &q)
 {
-	fail_on_test(q.alloc_bufs(node));
-
 	for (unsigned i = 0; i < q.g_buffers(); i++) {
 		buffer buf(q);
 		int ret;
@@ -1478,7 +1493,7 @@ static int setupUserPtr(struct node *node, cv4l_queue &q)
 			ret = buf.prepare_buf(node);
 			fail_on_test(!ret);
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
-				buf.s_userptr((char *)q.g_userptr(i, p) + 0xdeadbeef, p);
+				buf.s_userptr((char *)0x0eadb000 + buf.g_length(p), p);
 			ret = buf.prepare_buf(node);
 			fail_on_test(!ret);
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
@@ -1503,7 +1518,7 @@ static int setupUserPtr(struct node *node, cv4l_queue &q)
 			fail_on_test(!ret);
 
 			for (unsigned p = 0; p < buf.g_num_planes(); p++)
-				buf.s_userptr((char *)q.g_userptr(i, p) + 0xdeadbeef, p);
+				buf.s_userptr((char *)0x0eadb000 + buf.g_length(p), p);
 			ret = buf.qbuf(node);
 			fail_on_test(!ret);
 
@@ -1532,6 +1547,7 @@ static int setupUserPtr(struct node *node, cv4l_queue &q)
 int testUserPtr(struct node *node, struct node *node_m2m_cap, unsigned frame_count,
 		enum poll_mode pollmode)
 {
+	const __u32 filler = 0xdeadbeef;
 	bool can_stream = node->g_caps() & V4L2_CAP_STREAMING;
 	int type;
 	int ret;
@@ -1572,6 +1588,45 @@ int testUserPtr(struct node *node, struct node *node_m2m_cap, unsigned frame_cou
 		if (v4l_type_is_output(type))
 			stream_for_fmt(cur_fmt.g_pixelformat());
 
+		__u32 *buffers[q.g_buffers()][q.g_num_planes()];
+
+		/*
+		 * The alignment malloc uses depends on the gcc version and
+		 * architecture. Applications compiled for 64-bit all use a
+		 * 16 byte alignment. Applications compiled for 32-bit will
+		 * use an 8 byte alignment if glibc was compiled with gcc
+		 * version 6 or older, and 16 bytes when compiled with a newer
+		 * gcc. This is due to the support for _float128 that gcc
+		 * added in version 7 and that required this alignment change.
+		 *
+		 * Bottom line, in order to test user pointers the assumption
+		 * has to be that the DMA can handle writing just 8 bytes to a
+		 * page, since that's the worst case scenario.
+		 */
+		for (unsigned i = 0; i < q.g_buffers(); i++) {
+			for (unsigned p = 0; p < q.g_num_planes(); p++) {
+				/* ensure that len is a multiple of 4 */
+				__u32 len = ((q.g_length(p) + 3) & ~0x3) + 4 * 4096;
+				__u32 *m = static_cast<__u32 *>(malloc(len));
+
+				fail_on_test(!m);
+				fail_on_test((uintptr_t)m & 0x7);
+				for (__u32 *x = m; x < m + len / 4; x++)
+					*x = filler;
+				buffers[i][p] = m;
+				m = m + 2 * 4096 / 4;
+				/*
+				 * Put the start of the buffer at the last 8 bytes
+				 * of a page.
+				 */
+				m = (__u32 *)(((uintptr_t)m & ~0xfff) | 0xff8);
+				q.s_userptr(i, p, m);
+			}
+		}
+		// captureBufs() will update these values
+		memset(max_bytesused, 0, sizeof(max_bytesused));
+		memset(min_data_offset, 0xff, sizeof(min_data_offset));
+
 		fail_on_test(setupUserPtr(node, q));
 
 		if (node->codec_mask & STATEFUL_DECODER) {
@@ -1606,6 +1661,43 @@ int testUserPtr(struct node *node, struct node *node_m2m_cap, unsigned frame_cou
 			fail_on_test(m2m_q.reqbufs(node_m2m_cap, 0));
 			fail_on_test(!capture_count);
 		}
+		for (unsigned i = 0; i < q.g_buffers(); i++) {
+			for (unsigned p = 0; p < q.g_num_planes(); p++) {
+				__u32 buflen = (q.g_length(p) + 3U) & ~3U;
+				__u32 memlen = buflen + 4 * 4096;
+				__u32 *m = buffers[i][p];
+				__u32 *u = static_cast<__u32 *>(q.g_userptr(i, p));
+
+				for (__u32 *x = m; x < u; x++)
+					if (*x != filler)
+						fail("data at %zd bytes before start of the buffer was touched\n",
+						     (u - x) * 4);
+
+				unsigned data_offset = min_data_offset[p];
+				data_offset = (data_offset + 3U) & ~3U;
+				if (!v4l_type_is_output(type) && u[data_offset / 4] == filler)
+					fail("data at data_offset %u was untouched\n", data_offset);
+
+				unsigned used = max_bytesused[p];
+				// Should never happen
+				fail_on_test(!used);
+				used = (used + 3U) & ~3U;
+
+				for (__u32 *x = u + used / 4; x < u + buflen / 4; x++) {
+					if (*x == filler)
+						continue;
+					warn_once("data from max bytesused %u+%zd to length %u was touched in plane %u\n",
+						  used, (x - u) * 4 - used, buflen, p);
+					break;
+				}
+				for (__u32 *x = u + buflen / 4; x < m + memlen / 4; x++)
+					if (*x != filler)
+						fail("data at %zd bytes after the end of the buffer was touched\n",
+						     (x - (u + buflen / 4)) * 4);
+				free(m);
+				q.s_userptr(i, p, NULL);
+			}
+		}
 		stream_close();
 	}
 	return 0;
@@ -1660,7 +1752,7 @@ static int setupDmaBuf(struct node *expbuf_node, struct node *node,
 
 		fail_on_test(buf.qbuf(node, false));
 		for (unsigned p = 0; p < buf.g_num_planes(); p++) {
-			fail_on_test(buf.g_fd(p) != q.g_fd(i, p));;
+			fail_on_test(buf.g_fd(p) != q.g_fd(i, p));
 			fail_on_test(buf.g_length(p) != q.g_length(p));
 			if (v4l_type_is_output(q.g_type()))
 				fail_on_test(!buf.g_bytesused(p));
@@ -1699,7 +1791,7 @@ int testDmaBuf(struct node *expbuf_node, struct node *node, struct node *node_m2
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		else if (expbuf_node->g_caps() & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE;
-		else 
+		else
 			expbuf_type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 
 		cv4l_queue q(type, V4L2_MEMORY_DMABUF);
@@ -1903,7 +1995,7 @@ int testRequests(struct node *node, bool test_streaming)
 		node->g_fmt(fmt, type);
 		node->s_fmt(fmt, type);
 	}
-	
+
 	if (!(node->valid_buftypes & (1 << type))) {
 		fail_on_test(node->buf_caps & V4L2_BUF_CAP_SUPPORTS_REQUESTS);
 		return ENOTTY;
@@ -2162,7 +2254,7 @@ static int testBlockingDQBuf(struct node *node, cv4l_queue &q)
 		cv4l_buffer buf(q.g_type(), V4L2_MEMORY_MMAP);
 
 		node->dqbuf(buf);
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 	}
 
 	/* Wait for the child process to start and block */
@@ -2181,7 +2273,7 @@ static int testBlockingDQBuf(struct node *node, cv4l_queue &q)
 		 * be blocked by the DQBUF!
 		 */
 		node->streamoff(q.g_type());
-		exit(0);
+		std::exit(EXIT_SUCCESS);
 	}
 
 	int wstatus_streamoff = 0;
@@ -2219,7 +2311,7 @@ int testBlockingWait(struct node *node)
 
 		cv4l_queue q(type, V4L2_MEMORY_MMAP);
 		cv4l_queue m2m_q(v4l_type_invert(type), V4L2_MEMORY_MMAP);
-	
+
 		if (testSetupVbi(node, type))
 			continue;
 
@@ -2329,7 +2421,7 @@ static int testStreaming(struct node *node, unsigned frame_count)
 
 		unsigned field = cur_fmt.g_first_field(std);
 		cv4l_buffer buf(q);
-	
+
 		if (is_output)
 			stream_for_fmt(cur_fmt.g_pixelformat());
 
@@ -2474,9 +2566,7 @@ static void streamFmtRun(struct node *node, cv4l_fmt &fmt, unsigned frame_count,
 				compose.r.width, compose.r.height,
 				compose.r.left, compose.r.top);
 	}
-	if (!no_progress)
-		printf("\r");
-	printf("\t\t%s%sStride %u, Field %s%s: %s   \n",
+	printf("\r\t\t%s%sStride %u, Field %s%s: %s   \n",
 			s_crop, s_compose,
 			fmt.g_bytesperline(),
 			field2s(fmt.g_field()).c_str(),
@@ -2507,7 +2597,7 @@ static void streamFmt(struct node *node, __u32 pixelformat, __u32 w, __u32 h,
 	char hz[32] = "";
 
 	if (!frame_count)
-		frame_count = f ? 1.0 / fract2f(f) : 10;
+		frame_count = f ? static_cast<unsigned>(1.0 / fract2f(f)) : 10;
 	node->g_fmt(fmt);
 	fmt.s_pixelformat(pixelformat);
 	fmt.s_width(w);
@@ -2796,7 +2886,7 @@ void streamAllFormats(struct node *node, unsigned frame_count)
 						ss.max_width, ss.max_height,
 						frame_count);
 				restoreFormat(node);
-			}	
+			}
 			node->g_fmt(fmt);
 			if (fmt.g_width() != ss.min_width ||
 			    fmt.g_frame_height() != ss.min_height) {
@@ -2932,7 +3022,7 @@ void streamM2MAllFormats(struct node *node, unsigned frame_count)
 						   ss.max_width, ss.max_height,
 						   frame_count);
 				restoreFormat(node);
-			}	
+			}
 			streamM2MOutFormat(node, fmtdesc.pixelformat,
 					   fmt.g_width(), fmt.g_frame_height(),
 					   frame_count);
