@@ -30,6 +30,8 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <assert.h>
+
+#include "compiler.h"
 #include "v4l2-compliance.h"
 
 static const __u32 buftype2cap[] = {
@@ -97,7 +99,7 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt,
 		case V4L2_FRMIVAL_TYPE_CONTINUOUS:
 			if (sw->step.numerator != 1 || sw->step.denominator != 1)
 				return fail("invalid step for continuous frameinterval\n");
-			/* fallthrough */
+			fallthrough;
 		case V4L2_FRMIVAL_TYPE_STEPWISE:
 			if (frmival.index)
 				return fail("index must be 0 for stepwise/continuous frameintervals\n");
@@ -174,7 +176,7 @@ static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 				return ret;
 			if (ret == 0 && !(node->g_caps() & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_CAPTURE_MPLANE)))
 				return fail("found discrete framesizes when no video capture is supported\n");
-			cookie = ((__u64)pixfmt << 32) |
+			cookie = (static_cast<__u64>(pixfmt) << 32) |
 				 (frmsize.discrete.width << 16) |
 				 frmsize.discrete.height;
 			node->frmsizes.insert(cookie);
@@ -183,7 +185,7 @@ static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 		case V4L2_FRMSIZE_TYPE_CONTINUOUS:
 			if (frmsize.stepwise.step_width != 1 || frmsize.stepwise.step_height != 1)
 				return fail("invalid step_width/height for continuous framesize\n");
-			/* fallthrough */
+			fallthrough;
 		case V4L2_FRMSIZE_TYPE_STEPWISE:
 			if (frmsize.index)
 				return fail("index must be 0 for stepwise/continuous framesizes\n");
@@ -235,6 +237,7 @@ static int testEnumFormatsType(struct node *node, unsigned type)
 		memset(&fmtdesc, 0xff, sizeof(fmtdesc));
 		fmtdesc.type = type;
 		fmtdesc.index = f;
+		fmtdesc.mbus_code = 0;
 
 		ret = doioctl(node, VIDIOC_ENUM_FMT, &fmtdesc);
 		if (ret == ENOTTY)
@@ -305,8 +308,6 @@ int testEnumFormats(struct node *node)
 		case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
 		case V4L2_BUF_TYPE_SDR_CAPTURE:
 		case V4L2_BUF_TYPE_SDR_OUTPUT:
-		case V4L2_BUF_TYPE_META_CAPTURE:
-		case V4L2_BUF_TYPE_META_OUTPUT:
 			if (ret && (node->g_caps() & buftype2cap[type]))
 				return fail("%s cap set, but no %s formats defined\n",
 						buftype2s(type).c_str(), buftype2s(type).c_str());
@@ -314,6 +315,10 @@ int testEnumFormats(struct node *node)
 				return fail("%s cap not set, but %s formats defined\n",
 						buftype2s(type).c_str(), buftype2s(type).c_str());
 			break;
+		case V4L2_BUF_TYPE_META_CAPTURE:
+		case V4L2_BUF_TYPE_META_OUTPUT:
+			/* Metadata formats need not be present for the current input/output */
+			break;
 		default:
 			if (!ret)
 				return fail("Buffer type %s not allowed!\n", buftype2s(type).c_str());
@@ -402,7 +407,7 @@ static void createInvalidFmt(struct v4l2_format &fmt, struct v4l2_clip &clip, un
 	if (type == V4L2_BUF_TYPE_VIDEO_OVERLAY ||
 	    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_OVERLAY) {
 		memset(&clip, 0xff, sizeof(clip));
-		clip.next = (struct  v4l2_clip *)0xdeadbeef;
+		clip.next = (struct  v4l2_clip *)0x0eadbeef;
 		fmt.fmt.win.clipcount = 1;
 		fmt.fmt.win.clips = &clip;
 		fmt.fmt.win.bitmap = NULL;
@@ -445,7 +450,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 		fail_on_test(pix.bytesperline && pix.bytesperline < pix.width);
 		fail_on_test(!pix.sizeimage);
 		if (!node->is_m2m)
-			fail_on_test(testColorspace(node->has_inputs || node->has_outputs,
+			fail_on_test(testColorspace(!node->is_io_mc,
 						    pix.pixelformat, pix.colorspace,
 						    pix.ycbcr_enc, pix.quantization));
 		fail_on_test(pix.field == V4L2_FIELD_ANY);
@@ -461,7 +466,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 			return fail("pixelformat %08x (%s) for buftype %d not reported by ENUM_FMT\n",
 					pix_mp.pixelformat, fcc2s(pix_mp.pixelformat).c_str(), type);
 		if (!node->is_m2m)
-			fail_on_test(testColorspace(node->has_inputs || node->has_outputs,
+			fail_on_test(testColorspace(!node->is_io_mc,
 						    pix_mp.pixelformat, pix_mp.colorspace,
 						    pix_mp.ycbcr_enc, pix_mp.quantization));
 		fail_on_test(pix_mp.field == V4L2_FIELD_ANY);
@@ -537,7 +542,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 
 			fail_on_test(doioctl(node, VIDIOC_G_FBUF, &fb));
 			fail_on_test(!win.clips);
-			fail_on_test(win.clips->next != (void *)0xdeadbeef);
+			fail_on_test(win.clips->next != (void *)0x0eadbeef);
 			fail_on_test(win.clipcount != 1);
 			fail_on_test(r->left < 0 || r->top < 0);
 			fail_on_test((unsigned)r->left >= fb.fmt.width || (unsigned)r->top >= fb.fmt.height);
@@ -599,8 +604,6 @@ int testGetFormats(struct node *node)
 		case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
 		case V4L2_BUF_TYPE_SDR_CAPTURE:
 		case V4L2_BUF_TYPE_SDR_OUTPUT:
-		case V4L2_BUF_TYPE_META_CAPTURE:
-		case V4L2_BUF_TYPE_META_OUTPUT:
 			if (ret && (node->g_caps() & buftype2cap[type]))
 				return fail("%s cap set, but no %s formats defined\n",
 					buftype2s(type).c_str(), buftype2s(type).c_str());
@@ -608,6 +611,15 @@ int testGetFormats(struct node *node)
 				return fail("%s cap not set, but %s formats defined\n",
 					buftype2s(type).c_str(), buftype2s(type).c_str());
 			break;
+		case V4L2_BUF_TYPE_META_CAPTURE:
+		case V4L2_BUF_TYPE_META_OUTPUT:
+			if (ret && !node->buftype_pixfmts[type].empty())
+				return fail("%s G_FMT failed, but %s formats defined\n",
+					buftype2s(type).c_str(), buftype2s(type).c_str());
+			if (!ret && node->buftype_pixfmts[type].empty())
+				return fail("%s G_FMT success, but no %s formats defined\n",
+					buftype2s(type).c_str(), buftype2s(type).c_str());
+			break;
 		default:
 			/* ENUMFMT doesn't support other buftypes */
 			break;
@@ -649,10 +661,10 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 			return true;
 		printf("\t\tG_FMT:     %dx%d@%dx%d, %d, %x, %p, %d, %p, %x\n",
 			win1.w.width, win1.w.height, win1.w.left, win1.w.top, win1.field,
-			win1.chromakey, win1.clips, win1.clipcount, win1.bitmap, win1.global_alpha);
+			win1.chromakey, (void *)win1.clips, win1.clipcount, win1.bitmap, win1.global_alpha);
 		printf("\t\tTRY/S_FMT: %dx%d@%dx%d, %d, %x, %p, %d, %p, %x\n",
 			win2.w.width, win2.w.height, win2.w.left, win2.w.top, win2.field,
-			win2.chromakey, win2.clips, win2.clipcount, win2.bitmap, win2.global_alpha);
+			win2.chromakey, (void *)win2.clips, win2.clipcount, win2.bitmap, win2.global_alpha);
 		return false;
 	case V4L2_BUF_TYPE_VBI_CAPTURE:
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
@@ -681,7 +693,7 @@ int testTryFormats(struct node *node)
 	int result = 0;
 	int type;
 	int ret;
-	
+
 	for (type = 0; type <= V4L2_BUF_TYPE_LAST; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
@@ -1038,14 +1050,14 @@ int testSetFormats(struct node *node)
 	struct v4l2_format initial_fmts[V4L2_BUF_TYPE_LAST + 1];
 	int type;
 	int ret;
-	
+
 	for (type = 0; type <= V4L2_BUF_TYPE_LAST; type++) {
 		if (!(node->valid_buftypes & (1 << type)))
 			continue;
 
 		createInvalidFmt(fmt, clip, type);
 		doioctl(node, VIDIOC_G_FMT, &fmt);
-		
+
 		initial_fmts[type] = fmt;
 		createInvalidFmt(fmt_set, clip_set, type);
 		ret = doioctl(node, VIDIOC_S_FMT, &fmt_set);
@@ -1554,10 +1566,14 @@ int testCropping(struct node *node)
 	if (ret_cap && ret_out)
 		return ret_cap;
 
-	if (!ret_cap)
+	if (!ret_cap) {
+		fail_on_test(IS_ENCODER(node));
 		fail_on_test(testBasicCrop(node, V4L2_BUF_TYPE_VIDEO_CAPTURE));
-	if (!ret_out)
+	}
+	if (!ret_out) {
+		fail_on_test(IS_DECODER(node));
 		fail_on_test(testBasicCrop(node, V4L2_BUF_TYPE_VIDEO_OUTPUT));
+	}
 
 	return 0;
 }
@@ -1645,10 +1661,14 @@ int testComposing(struct node *node)
 	if (ret_cap && ret_out)
 		return ret_cap;
 
-	if (!ret_cap)
+	if (!ret_cap) {
+		fail_on_test(IS_ENCODER(node));
 		fail_on_test(testBasicCompose(node, V4L2_BUF_TYPE_VIDEO_CAPTURE));
-	if (!ret_out)
+	}
+	if (!ret_out) {
+		fail_on_test(IS_DECODER(node));
 		fail_on_test(testBasicCompose(node, V4L2_BUF_TYPE_VIDEO_OUTPUT));
+	}
 
 	return 0;
 }
@@ -1710,7 +1730,7 @@ static int testBasicScaling(struct node *node, const struct v4l2_format &cur)
 		compose_is_crop = compose_w == crop_w &&
 				  compose_h == crop_h;
 
-	cookie = ((__u64)v4l_format_g_pixelformat(&fmt) << 32) |
+	cookie = (static_cast<__u64>(v4l_format_g_pixelformat(&fmt)) << 32) |
 		  (v4l_format_g_width(&fmt) << 16) |
 		  v4l_format_g_height(&fmt);
 	if (node->can_output) {
@@ -1750,7 +1770,7 @@ static int testBasicScaling(struct node *node, const struct v4l2_format &cur)
 	if (compose_is_crop)
 		compose_is_crop = sel_compose.r.width == sel_crop.r.width &&
 				  sel_compose.r.height == sel_crop.r.height;
-	cookie = ((__u64)v4l_format_g_pixelformat(&fmt) << 32) |
+	cookie = (static_cast<__u64>(v4l_format_g_pixelformat(&fmt)) << 32) |
 		  (v4l_format_g_width(&fmt) << 16) |
 		  v4l_format_g_height(&fmt);
 	if (node->can_output) {
@@ -1773,6 +1793,47 @@ static int testBasicScaling(struct node *node, const struct v4l2_format &cur)
 	return 0;
 }
 
+static int testM2MScaling(struct node *node)
+{
+	struct v4l2_selection sel_compose = {
+		V4L2_BUF_TYPE_VIDEO_CAPTURE,
+		V4L2_SEL_TGT_COMPOSE,
+		0,
+		{ 1, 1, 0, 0 }
+	};
+	__u32 cap_type = node->g_type();
+	__u32 out_type = v4l_type_invert(cap_type);
+	cv4l_fmt out_fmt, cap_fmt, fmt;
+
+	fail_on_test(node->g_fmt(out_fmt, out_type));
+	out_fmt.s_width(1);
+	out_fmt.s_height(1);
+	fail_on_test(node->s_fmt(out_fmt, out_type));
+
+	fail_on_test(node->g_fmt(cap_fmt, cap_type));
+	cap_fmt.s_width(out_fmt.g_width());
+	cap_fmt.s_height(out_fmt.g_height());
+	fail_on_test(node->s_fmt(cap_fmt, cap_type));
+	fmt = cap_fmt;
+	fmt.s_width(0x4000);
+	fmt.s_height(0x4000);
+	fail_on_test(node->s_fmt(fmt, cap_type));
+	if (!doioctl(node, VIDIOC_G_SELECTION, &sel_compose)) {
+		sel_compose.r.width = fmt.g_width();
+		sel_compose.r.height = fmt.g_height();
+		doioctl(node, VIDIOC_S_SELECTION, &sel_compose);
+		doioctl(node, VIDIOC_G_SELECTION, &sel_compose);
+		if (sel_compose.r.width > cap_fmt.g_width() ||
+		    sel_compose.r.height > cap_fmt.g_height())
+			node->can_scale = true;
+	} else {
+		if (fmt.g_width() > cap_fmt.g_width() ||
+		    fmt.g_height() > cap_fmt.g_height())
+			node->can_scale = true;
+	}
+	return node->can_scale ? 0 : ENOTTY;
+}
+
 int testScaling(struct node *node)
 {
 	struct v4l2_format fmt;
@@ -1780,6 +1841,17 @@ int testScaling(struct node *node)
 	if (!node->is_video)
 		return ENOTTY;
 	node->can_scale = false;
+
+	/*
+	 * Encoders do not have a scaler. Decoders might, but it is
+	 * very hard to detect this for any decoder but JPEG.
+	 */
+	if (node->codec_mask && node->codec_mask != JPEG_DECODER)
+		return ENOTTY;
+
+	if (node->is_m2m)
+		return testM2MScaling(node);
+
 	if (node->can_capture) {
 		v4l_format_init(&fmt, node->is_planar ?
 			V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
