@@ -79,7 +79,7 @@ static int checkTuner(struct node *node, const struct v4l2_tuner &tuner,
 		unsigned t, v4l2_std_id std)
 {
 	bool valid_modes[5] = { true, false, false, false, false };
-	bool tv = node->is_video || node->is_vbi;
+	bool tv = node->is_video || node->is_vbi || node->is_meta;
 	bool hwseek_caps = tuner.capability & (V4L2_TUNER_CAP_HWSEEK_BOUNDED |
 			V4L2_TUNER_CAP_HWSEEK_WRAP | V4L2_TUNER_CAP_HWSEEK_PROG_LIM);
 	unsigned type = tv ? V4L2_TUNER_ANALOG_TV : V4L2_TUNER_RADIO;
@@ -228,12 +228,12 @@ int testTunerFreq(struct node *node)
 
 	for (t = 0; t < node->tuners; t++) {
 		struct v4l2_tuner tuner = { 0 };
-		
+
 		tuner.index = t;
 		ret = doioctl(node, VIDIOC_G_TUNER, &tuner);
 		if (ret)
 			return fail("could not get tuner %d\n", t);
-		last_type = (enum v4l2_tuner_type)tuner.type;
+		last_type = static_cast<enum v4l2_tuner_type>(tuner.type);
 		memset(&freq, 0, sizeof(freq));
 		freq.tuner = t;
 		ret = doioctl(node, VIDIOC_G_FREQUENCY, &freq);
@@ -259,7 +259,7 @@ int testTunerFreq(struct node *node)
 			warn("returned tuner %d frequency out of range (%d not in [%d...%d])\n",
 					t, freq.frequency, tuner.rangelow, tuner.rangehigh);
 
-		freq.type = (enum v4l2_tuner_type)0;
+		freq.type = static_cast<enum v4l2_tuner_type>(0);
 		ret = doioctl(node, VIDIOC_S_FREQUENCY, &freq);
 		if (ret != EINVAL)
 			return fail("did not return EINVAL when passed tuner type 0\n");
@@ -317,7 +317,7 @@ int testTunerHwSeek(struct node *node)
 
 	for (t = 0; t < node->tuners; t++) {
 		struct v4l2_tuner tuner = { 0 };
-		
+
 		tuner.index = t;
 		ret = doioctl(node, VIDIOC_G_TUNER, &tuner);
 		if (ret)
@@ -466,6 +466,10 @@ int testInput(struct node *node)
 	if (!node->inputs && node->has_inputs)
 		return fail("no inputs found, but input capabilities set\n");
 	fail_on_test(node->is_m2m && node->inputs > 1);
+	if (node->is_io_mc) {
+		fail_on_test(!node->is_video && !node->is_meta);
+		fail_on_test(node->inputs != 1);
+	}
 	return 0;
 }
 
@@ -674,7 +678,7 @@ int testModulatorFreq(struct node *node)
 
 	for (m = 0; m < node->modulators; m++) {
 		struct v4l2_modulator modulator;
-		
+
 		modulator.index = m;
 		memset(modulator.reserved, 0, sizeof(modulator.reserved));
 		ret = doioctl(node, VIDIOC_G_MODULATOR, &modulator);
@@ -836,6 +840,10 @@ int testOutput(struct node *node)
 	if (!node->outputs && node->has_outputs)
 		return fail("no outputs found, but output capabilities set\n");
 	fail_on_test(node->is_m2m && node->outputs > 1);
+	if (node->is_io_mc) {
+		fail_on_test(!node->is_video && !node->is_meta);
+		fail_on_test(node->outputs != 1);
+	}
 	return 0;
 }
 
