@@ -395,7 +395,7 @@ struct target_info {
 
 static target_info targets[] = {
 	{ V4L2_SEL_TGT_CROP, true },
-	{ V4L2_SEL_TGT_CROP_DEFAULT, false, true },
+	{ V4L2_SEL_TGT_CROP_DEFAULT, true, true },
 	{ V4L2_SEL_TGT_CROP_BOUNDS, true, true },
 	{ V4L2_SEL_TGT_NATIVE_SIZE, true },
 	{ V4L2_SEL_TGT_COMPOSE, true },
@@ -411,7 +411,7 @@ int testSubDevSelection(struct node *node, unsigned which, unsigned pad)
 	struct v4l2_subdev_selection s_sel;
 	struct v4l2_subdev_crop crop;
 	bool is_sink = node->pads[pad].flags & MEDIA_PAD_FL_SINK;
-	bool have_sel;
+	bool have_sel = false;
 	int ret;
 
 	targets[V4L2_SEL_TGT_NATIVE_SIZE].readonly = is_sink;
@@ -494,6 +494,5 @@ int testSubDevSelection(struct node *node, unsigned which, unsigned pad)
 		fail_on_test(s_sel.r.height != sel.r.height);
 	}
 
-	fail_on_test(!have_sel);
-	return 0;
+	return have_sel ? 0 : ENOTTY;
 }
