@@ -1,7 +1,9 @@
+#include <cstdlib>
+#include <cstring>
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 #include <inttypes.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -122,7 +124,7 @@ static __u8 toggle_hdr_md_flags;
 #define HDR_MD_SMPTE_2084	(1 << 2)
 #define HDR_MD_HLG		(1 << 3)
 
-void edid_usage(void)
+void edid_usage()
 {
 	printf("\nEDID options:\n"
 	       "  --set-edid pad=<pad>[,type=<type>|file=<file>][,format=<fmt>][modifiers]\n"
@@ -233,9 +235,9 @@ static void edid_add_block(struct v4l2_edid *e)
 		fprintf(stderr, "edid file error: too long\n");
 		free(e->edid);
 		e->edid = NULL;
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
-	e->edid = (unsigned char *)realloc(e->edid, e->blocks * 128);
+	e->edid = static_cast<unsigned char *>(realloc(e->edid, e->blocks * 128));
 }
 
 static void read_edid_file(FILE *f, struct v4l2_edid *e)
@@ -593,7 +595,7 @@ static void print_edid_mods(const struct v4l2_edid *e)
 
 		printf("\nHDMI Vendor-Specific Data Block\n");
 		if (v)
-			printf("  Max TMDS Clock:          %u MHz\n", v * 5); 
+			printf("  Max TMDS Clock:          %u MHz\n", v * 5);
 		printf("  Physical Address:        %x.%x.%x.%x\n",
 		       pa >> 12, (pa >> 8) & 0xf, (pa >> 4) & 0xf, pa & 0xf);
 		if (len >= 6) {
@@ -645,7 +647,7 @@ static void print_edid_mods(const struct v4l2_edid *e)
 
 		printf("\nHDMI Forum Vendor-Specific Data Block\n");
 		if (v)
-			printf("  Max TMDS Character Rate: %u MHz\n", v * 5); 
+			printf("  Max TMDS Character Rate: %u MHz\n", v * 5);
 		v = e->edid[loc + 1];
 		printf("  SCDC Present:            %s\n", (v & HF_VSDB_SCSD_PRESENT) ? "yes" : "no");
 	}
@@ -710,7 +712,7 @@ static unsigned short parse_phys_addr(const char *value)
 {
 	unsigned p1, p2, p3, p4;
 
-	if (!strchr(value, '.'))
+	if (!std::strchr(value, '.'))
 		return strtoul(value, NULL, 0);
 	if (sscanf(value, "%x.%x.%x.%x", &p1, &p2, &p3, &p4) != 4) {
 		fprintf(stderr, "Expected a physical address of the form x.x.x.x\n");
@@ -986,13 +988,13 @@ void edid_cmd(int ch, char *optarg)
 			if (opt == -1) {
 				fprintf(stderr, "Invalid suboptions specified\n");
 				edid_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			if (value == NULL && opt <= 8) {
 				fprintf(stderr, "No value given to suboption <%s>\n",
 					subopts[opt]);
 				edid_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			switch (opt) {
 			case 0:
@@ -1020,11 +1022,11 @@ void edid_cmd(int ch, char *optarg)
 					sedid.blocks = sizeof(hdmi_edid) / 128;
 				} else {
 					edid_usage();
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				if (file_in) {
 					fprintf(stderr, "The edid and file options can't be used together.\n");
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				break;
 			case 3:
@@ -1032,7 +1034,7 @@ void edid_cmd(int ch, char *optarg)
 					file_in = value;
 					if (sedid.edid) {
 						fprintf(stderr, "The edid and file options can't be used together.\n");
-						exit(1);
+						std::exit(EXIT_FAILURE);
 					}
 				}
 				break;
@@ -1043,7 +1045,7 @@ void edid_cmd(int ch, char *optarg)
 					sformat = RAW;
 				} else {
 					edid_usage();
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				break;
 			case 5:
@@ -1109,7 +1111,7 @@ void edid_cmd(int ch, char *optarg)
 			case 56: toggle_speaker3_flags |= SPEAKER3_TPLS_TPRS; break;
 			default:
 				edid_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		break;
@@ -1145,7 +1147,7 @@ void edid_cmd(int ch, char *optarg)
 				gedid.start_block = strtoul(value, 0, 0);
 				if (gedid.start_block > 255) {
 					fprintf(stderr, "startblock %d too large, max 255\n", gedid.start_block);
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				break;
 			case 2:
@@ -1160,7 +1162,7 @@ void edid_cmd(int ch, char *optarg)
 					gformat = CARRAY;
 				} else {
 					edid_usage();
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				break;
 			case 4:
@@ -1169,7 +1171,7 @@ void edid_cmd(int ch, char *optarg)
 				break;
 			default:
 				edid_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		if (gedid.start_block + gedid.blocks > 256)
@@ -1209,7 +1211,7 @@ void edid_set(cv4l_fd &_fd)
 			if (!fin) {
 				fprintf(stderr, "Failed to open %s: %s\n", file_in,
 						strerror(errno));
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		if (fin) {
@@ -1217,7 +1219,7 @@ void edid_set(cv4l_fd &_fd)
 			if (sedid.blocks == 0) {
 				fprintf(stderr, "%s contained an empty EDID, ignoring.\n",
 						file_in ? file_in : "stdin");
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		if (toggle_cta861_hdr_flags || phys_addr >= 0) {
@@ -1329,10 +1331,10 @@ void edid_get(cv4l_fd &_fd)
 			if (!fout) {
 				fprintf(stderr, "Failed to open %s: %s\n", file_out,
 						strerror(errno));
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
-		gedid.edid = (unsigned char *)malloc(gedid.blocks * 128);
+		gedid.edid = static_cast<unsigned char *>(malloc(gedid.blocks * 128));
 		if (doioctl(fd, VIDIOC_G_EDID, &gedid) == 0) {
 			if (options[OptFixEdidChecksums])
 				fix_edid(&gedid);
@@ -1344,7 +1346,7 @@ void edid_get(cv4l_fd &_fd)
 	}
 	if (options[OptInfoEdid]) {
 		info_edid.blocks = 2;
-		info_edid.edid = (unsigned char *)malloc(info_edid.blocks * 128);
+		info_edid.edid = static_cast<unsigned char *>(malloc(info_edid.blocks * 128));
 		if (doioctl(fd, VIDIOC_G_EDID, &info_edid) == 0)
 			print_edid_mods(&info_edid);
 		free(info_edid.edid);
