@@ -28,7 +28,7 @@ static const char *fb_device;
 static std::vector<struct v4l2_clip> clips;
 static std::vector<struct v4l2_rect> bitmap_rects;
 
-void overlay_usage(void)
+void overlay_usage()
 {
 	printf("\nVideo Overlay options:\n"
 	       "  --list-formats-overlay\n"
@@ -395,19 +395,19 @@ static void do_try_set_overlay(struct v4l2_format &fmt, int fd)
 	if (((set_overlay_fmt & FmtWidth) && win.w.width != overlay_fmt.fmt.win.w.width) ||
 	    ((set_overlay_fmt & FmtHeight) && win.w.height != overlay_fmt.fmt.win.w.height))
 		keep_old_bitmap = keep_old_clip = false;
-	if (options[OptClearBitmap] || bitmap_rects.size())
+	if (options[OptClearBitmap] || !bitmap_rects.empty())
 		keep_old_bitmap = false;
-	if (options[OptClearClips] || clips.size())
+	if (options[OptClearClips] || !clips.empty())
 		keep_old_clip = false;
 
 	win.bitmap = NULL;
 	if (keep_old_bitmap) {
-		bitmap = (unsigned char *)calloc(1, stride * win.w.height);
+		bitmap = static_cast<unsigned char *>(calloc(1, stride * win.w.height));
 		win.bitmap = bitmap;
 	}
 	if (keep_old_clip) {
 		if (win.clipcount)
-			cliplist = (struct v4l2_clip *)malloc(win.clipcount * sizeof(*cliplist));
+			cliplist = static_cast<struct v4l2_clip *>(malloc(win.clipcount * sizeof(*cliplist)));
 		win.clips = cliplist;
 	}
 	if (keep_old_clip || keep_old_bitmap)
@@ -428,14 +428,14 @@ static void do_try_set_overlay(struct v4l2_format &fmt, int fd)
 		win.w.height = overlay_fmt.fmt.win.w.height;
 	if (set_overlay_fmt & FmtField)
 		win.field = overlay_fmt.fmt.win.field;
-	if (clips.size()) {
+	if (!clips.empty()) {
 		win.clipcount = clips.size();
 		win.clips = &clips[0];
 	}
-	if (bitmap_rects.size()) {
+	if (!bitmap_rects.empty()) {
 		free(bitmap);
 		stride = (win.w.width + 7) / 8;
-		bitmap = (unsigned char *)calloc(1, stride * win.w.height);
+		bitmap = static_cast<unsigned char *>(calloc(1, stride * win.w.height));
 		win.bitmap = bitmap;
 		for (unsigned i = 0; i < bitmap_rects.size(); i++) {
 			const v4l2_rect &r = bitmap_rects[i];
@@ -471,7 +471,7 @@ void overlay_set(cv4l_fd &_fd)
 
 	if ((options[OptSetOverlayFormat] || options[OptTryOverlayFormat]) &&
 			(set_overlay_fmt || options[OptClearClips] || options[OptClearBitmap] ||
-			 bitmap_rects.size() || clips.size())) {
+			 !bitmap_rects.empty() || !clips.empty())) {
 		struct v4l2_format fmt;
 
 		memset(&fmt, 0, sizeof(fmt));
@@ -518,8 +518,8 @@ void overlay_get(cv4l_fd &_fd)
 			unsigned stride = (fmt.fmt.win.w.width + 7) / 8;
 
 			if (fmt.fmt.win.clipcount)
-				fmt.fmt.win.clips = (struct v4l2_clip *)malloc(fmt.fmt.win.clipcount * sizeof(clips[0]));
-			bitmap = (unsigned char *)calloc(1, stride * fmt.fmt.win.w.height);
+				fmt.fmt.win.clips = static_cast<struct v4l2_clip *>(malloc(fmt.fmt.win.clipcount * sizeof(clips[0])));
+			bitmap = static_cast<unsigned char *>(calloc(1, stride * fmt.fmt.win.w.height));
 			fmt.fmt.win.bitmap = bitmap;
 			doioctl(fd, VIDIOC_G_FMT, &fmt);
 			printfmt(fd, fmt);
