@@ -13,6 +13,7 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
+#include <cstdlib>
 
 #include "v4l2-ctl.h"
 
@@ -25,7 +26,7 @@ static unsigned set_dv_timing_opts;
 static __u32 list_dv_timings_pad;
 static __u32 dv_timings_cap_pad;
 
-void stds_usage(void)
+void stds_usage()
 {
 	printf("\nStandards/Timings options:\n"
 	       "  --list-standards   display supported video standards [VIDIOC_ENUMSTD]\n"
@@ -102,7 +103,7 @@ static v4l2_std_id parse_pal(const char *pal)
 	}
 	fprintf(stderr, "pal specifier not recognised\n");
 	stds_usage();
-	exit(1);
+	std::exit(EXIT_FAILURE);
 }
 
 static v4l2_std_id parse_secam(const char *secam)
@@ -124,7 +125,7 @@ static v4l2_std_id parse_secam(const char *secam)
 	}
 	fprintf(stderr, "secam specifier not recognised\n");
 	stds_usage();
-	exit(1);
+	std::exit(EXIT_FAILURE);
 }
 
 static v4l2_std_id parse_ntsc(const char *ntsc)
@@ -141,7 +142,7 @@ static v4l2_std_id parse_ntsc(const char *ntsc)
 	}
 	fprintf(stderr, "ntsc specifier not recognised\n");
 	stds_usage();
-	exit(1);
+	std::exit(EXIT_FAILURE);
 }
 
 enum timing_opts {
@@ -204,13 +205,13 @@ static int parse_timing_subopt(char **subopt_str, int *value)
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
 		stds_usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	if (opt_str == NULL && opt != CVT && opt != GTF && opt != CLEAR) {
 		fprintf(stderr, "No value given to suboption <%s>\n",
 				subopt_list[opt]);
 		stds_usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	if (opt_str)
@@ -269,7 +270,7 @@ static void get_cvt_gtf_timings(char *subopt, int standard,
 
 	if (!timings_valid) {
 		stds_usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 }
 
@@ -356,7 +357,7 @@ static void parse_dv_bt_timings(char *optarg, struct v4l2_dv_timings *dv_timings
 			break;
 		default:
 			stds_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		set_dv_timing_opts |= 1 << opt;
 	}
@@ -379,7 +380,7 @@ static void print_dv_timings(const struct v4l2_dv_timings *t)
 		if (options[OptConcise]) {
 			printf("\t%dx%d%c%.2f %s\n", bt->width, bt->height,
 				bt->interlaced ? 'i' : 'p',
-				(double)bt->pixelclock /
+				static_cast<double>(bt->pixelclock) /
 					(tot_width * (tot_height / (bt->interlaced ? 2 : 1))),
 				dvflags2s(bt->vsync, bt->flags).c_str());
 			break;
@@ -399,10 +400,10 @@ static void print_dv_timings(const struct v4l2_dv_timings *t)
 		printf("\tPixelclock: %lld Hz", bt->pixelclock);
 		if (bt->width && bt->height) {
 			if (bt->interlaced)
-				printf(" (%.2f fields per second)", (double)bt->pixelclock /
+				printf(" (%.2f fields per second)", static_cast<double>(bt->pixelclock) /
 					(tot_width * (tot_height / 2)));
 			else
-				printf(" (%.2f frames per second)", (double)bt->pixelclock /
+				printf(" (%.2f frames per second)", static_cast<double>(bt->pixelclock) /
 					(tot_width * tot_height));
 		}
 		printf("\n");
@@ -464,7 +465,7 @@ void stds_cmd(int ch, char *optarg)
 		} else {
 			fprintf(stderr, "Unknown standard '%s'\n", optarg);
 			stds_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptSetDvBtTimings:
@@ -495,7 +496,7 @@ void stds_set(cv4l_fd &_fd)
 			}
 		}
 		if (doioctl(fd, VIDIOC_S_STD, &standard) == 0)
-			printf("Standard set to %08llx\n", (unsigned long long)standard);
+			printf("Standard set to %08llx\n", static_cast<unsigned long long>(standard));
 	}
 
 	if (options[OptSetDvBtTimings]) {
@@ -585,7 +586,7 @@ void stds_get(cv4l_fd &_fd)
 
 	if (options[OptGetStandard]) {
 		if (doioctl(fd, VIDIOC_G_STD, &standard) == 0) {
-			printf("Video Standard = 0x%08llx\n", (unsigned long long)standard);
+			printf("Video Standard = 0x%08llx\n", static_cast<unsigned long long>(standard));
 			printf("\t%s\n", std2s(standard, "\n\t").c_str());
 		}
 	}
@@ -624,7 +625,7 @@ void stds_get(cv4l_fd &_fd)
 
         if (options[OptQueryStandard]) {
 		if (doioctl(fd, VIDIOC_QUERYSTD, &standard) == 0) {
-			printf("Video Standard = 0x%08llx\n", (unsigned long long)standard);
+			printf("Video Standard = 0x%08llx\n", static_cast<unsigned long long>(standard));
 			printf("\t%s\n", std2s(standard, "\n\t").c_str());
 		}
 	}
@@ -647,12 +648,12 @@ void stds_list(cv4l_fd &_fd)
 		while (test_ioctl(fd, VIDIOC_ENUMSTD, &vs) >= 0) {
 			if (options[OptConcise]) {
 				printf("\t%2d: 0x%016llX %s\n", vs.index,
-						(unsigned long long)vs.id, vs.name);
+						static_cast<unsigned long long>(vs.id), vs.name);
 			} else {
 				if (vs.index)
 					printf("\n");
 				printf("\tIndex       : %d\n", vs.index);
-				printf("\tID          : 0x%016llX\n", (unsigned long long)vs.id);
+				printf("\tID          : 0x%016llX\n", static_cast<unsigned long long>(vs.id));
 				printf("\tName        : %s\n", vs.name);
 				printf("\tFrame period: %d/%d\n",
 						vs.frameperiod.numerator,
