@@ -12,7 +12,8 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <dirent.h>
-#include <math.h>
+#include <cmath>
+#include <cstdlib>
 
 #include "v4l2-ctl.h"
 
@@ -26,7 +27,7 @@ static struct v4l2_hw_freq_seek freq_seek; /* freq-seek */
 static double low, high;		/* freq-seek frequency range */
 static int mode = V4L2_TUNER_MODE_STEREO;  /* set audio mode */
 
-void tuner_usage(void)
+void tuner_usage()
 {
 	printf("\nTuner/Modulator options:\n"
 	       "  -F, --get-freq     query the frequency [VIDIOC_G_FREQUENCY]\n"
@@ -198,7 +199,7 @@ static void parse_freq_seek(char *optarg, struct v4l2_hw_freq_seek &seek)
 			break;
 		default:
 			tuner_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 }
@@ -223,7 +224,7 @@ void tuner_cmd(int ch, char *optarg)
 		else {
 			fprintf(stderr, "Unknown audio mode\n");
 			tuner_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptSetModulator:
@@ -245,7 +246,7 @@ void tuner_cmd(int ch, char *optarg)
 		else {
 			fprintf(stderr, "Unknown txsubchans value\n");
 			tuner_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptFreqSeek:
@@ -422,7 +423,8 @@ void tuner_get(cv4l_fd &_fd)
 				     vt.rangelow / 16.0, vt.rangehigh / 16.0);
 
 			if (vt.type != V4L2_TUNER_SDR && vt.type != V4L2_TUNER_RF) {
-				printf("\tSignal strength/AFC  : %d%%/%d\n", (int)((vt.signal / 655.35)+0.5), vt.afc);
+				printf("\tSignal strength/AFC  : %ld%%/%d\n",
+				       lround(vt.signal / 655.35), vt.afc);
 				printf("\tCurrent audio mode   : %s\n", audmode2s(vt.audmode));
 				printf("\tAvailable subchannels: %s\n", rxsubchans2s(vt.rxsubchans).c_str());
 			}
