@@ -1,7 +1,8 @@
+#include <cstring>
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 #include <inttypes.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -13,7 +14,9 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
+#include <cstdlib>
 
+#include "compiler.h"
 #include "v4l2-ctl.h"
 
 static struct v4l2_format sliced_fmt;	  /* set_format/get_format for sliced VBI */
@@ -21,7 +24,7 @@ static struct v4l2_format sliced_fmt_out; /* set_format/get_format for sliced VB
 static struct v4l2_format raw_fmt;	  /* set_format/get_format for VBI */
 static struct v4l2_format raw_fmt_out;	  /* set_format/get_format for VBI output */
 
-void vbi_usage(void)
+void vbi_usage()
 {
 	printf("\nVBI Formats options:\n"
 	       "  --get-sliced-vbi-cap\n"
@@ -87,17 +90,17 @@ void vbi_cmd(int ch, char *optarg)
 	case OptSetSlicedVbiOutFormat:
 	case OptTrySlicedVbiOutFormat:
 		sliced = &sliced_fmt_out;
-		/* fall through */
+		fallthrough;
 	case OptSetSlicedVbiFormat:
 	case OptTrySlicedVbiFormat:
 		sliced->fmt.sliced.service_set = 0;
 		if (optarg[0] == 0) {
 			fprintf(stderr, "empty string\n");
 			vbi_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		while (*optarg) {
-			subs = strchr(optarg, ',');
+			subs = std::strchr(optarg, ',');
 			if (subs)
 				*subs = 0;
 
@@ -124,13 +127,13 @@ void vbi_cmd(int ch, char *optarg)
 		if (found_off && sliced->fmt.sliced.service_set) {
 			fprintf(stderr, "Sliced VBI mode 'off' cannot be combined with other modes\n");
 			vbi_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptSetVbiOutFormat:
 	case OptTryVbiOutFormat:
 		raw = &raw_fmt_out;
-		/* fall through */
+		fallthrough;
 	case OptSetVbiFormat:
 	case OptTryVbiFormat:
 		subs = optarg;
