@@ -287,7 +287,7 @@ static struct option long_options[] = {
 	{0, 0, 0, 0}
 };
 
-static void usage_all(void)
+static void usage_all()
 {
        common_usage();
        tuner_usage();
@@ -299,6 +299,7 @@ static void usage_all(void)
        vbi_usage();
        sdr_usage();
        meta_usage();
+       subdev_usage();
        selection_usage();
        misc_usage();
        streaming_usage();
@@ -388,14 +389,14 @@ static bool is_rgb_or_hsv(__u32 pixelformat)
 
 static std::string printfmtname(int fd, __u32 type, __u32 pixfmt)
 {
-	struct v4l2_fmtdesc fmt;
+	struct v4l2_fmtdesc fmt = {};
 	std::string s(" (");
 
 	fmt.index = 0;
 	fmt.type = type;
 	while (test_ioctl(fd, VIDIOC_ENUM_FMT, &fmt) >= 0) {
 		if (fmt.pixelformat == pixfmt)
-			return s + (const char *)fmt.description + ")";
+			return s + reinterpret_cast<const char *>(fmt.description) + ")";
 		fmt.index++;
 	}
 	return "";
@@ -475,7 +476,7 @@ void printfmt(int fd, const struct v4l2_format &vfmt)
 			}
 		printf("\tClip Bitmap : %s", vfmt.fmt.win.bitmap ? "Yes, " : "No\n");
 		if (vfmt.fmt.win.bitmap) {
-			unsigned char *bitmap = (unsigned char *)vfmt.fmt.win.bitmap;
+			unsigned char *bitmap = static_cast<unsigned char *>(vfmt.fmt.win.bitmap);
 			unsigned stride = (vfmt.fmt.win.w.width + 7) / 8;
 			unsigned cnt = 0;
 
@@ -492,7 +493,7 @@ void printfmt(int fd, const struct v4l2_format &vfmt)
 		printf("\tSampling Rate   : %u Hz\n", vfmt.fmt.vbi.sampling_rate);
 		printf("\tOffset          : %u samples (%g secs after leading edge)\n",
 				vfmt.fmt.vbi.offset,
-				(double)vfmt.fmt.vbi.offset / (double)vfmt.fmt.vbi.sampling_rate);
+				static_cast<double>(vfmt.fmt.vbi.offset) / static_cast<double>(vfmt.fmt.vbi.sampling_rate));
 		printf("\tSamples per Line: %u\n", vfmt.fmt.vbi.samples_per_line);
 		printf("\tSample Format   : '%s'\n", fcc2s(vfmt.fmt.vbi.sample_format).c_str());
 		printf("\tStart 1st Field : %u\n", vfmt.fmt.vbi.start[0]);
@@ -606,7 +607,7 @@ void print_frmival(const struct v4l2_frmivalenum &frmival, const char *prefix)
 void print_video_formats(cv4l_fd &fd, __u32 type)
 {
 	cv4l_disable_trace dt(fd);
-	struct v4l2_fmtdesc fmt;
+	struct v4l2_fmtdesc fmt = {};
 
 	printf("\tType: %s\n\n", buftype2s(type).c_str());
 	if (fd.enum_fmt(fmt, true, 0, type))
@@ -623,7 +624,7 @@ void print_video_formats(cv4l_fd &fd, __u32 type)
 void print_video_formats_ext(cv4l_fd &fd, __u32 type)
 {
 	cv4l_disable_trace dt(fd);
-	struct v4l2_fmtdesc fmt;
+	struct v4l2_fmtdesc fmt = {};
 	struct v4l2_frmsizeenum frmsize;
 	struct v4l2_frmivalenum frmival;
 
@@ -656,7 +657,7 @@ void print_video_formats_ext(cv4l_fd &fd, __u32 type)
 
 int parse_subopt(char **subs, const char * const *subopts, char **value)
 {
-	int opt = getsubopt(subs, (char * const *)subopts, value);
+	int opt = getsubopt(subs, const_cast<char * const *>(subopts), value);
 
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
@@ -744,11 +745,13 @@ __u32 parse_quantization(const char *s)
 
 int parse_fmt(char *optarg, __u32 &width, __u32 &height, __u32 &pixelformat,
 	      __u32 &field, __u32 &colorspace, __u32 &xfer_func, __u32 &ycbcr,
-	      __u32 &quantization, __u32 &flags, __u32 *bytesperline)
+	      __u32 &quantization, __u32 &flags, __u32 *bytesperline,
+	      __u32 *sizeimage)
 {
 	char *value, *subs;
 	int fmts = 0;
 	unsigned bpl_index = 0;
+	unsigned sizeimage_index = 0;
 	bool be_pixfmt;
 
 	field = V4L2_FIELD_ANY;
@@ -767,6 +770,7 @@ int parse_fmt(char *optarg, __u32 &width, __u32 &height, __u32 &pixelformat,
 			"premul-alpha",
 			"quantization",
 			"xfer",
+			"sizeimage",
 			NULL
 		};
 
@@ -791,7 +795,7 @@ int parse_fmt(char *optarg, __u32 &width, __u32 &height, __u32 &pixelformat,
 				pixelformat = strtol(value, 0L, 0);
 			} else {
 				fprintf(stderr, "The pixelformat '%s' is invalid\n", value);
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			fmts |= FmtPixelFormat;
 			break;
@@ -835,6 +839,11 @@ int parse_fmt(char *optarg, __u32 &width, __u32 &height, __u32 &pixelformat,
 			xfer_func = parse_xfer_func(value);
 			fmts |= FmtXferFunc;
 			break;
+		case 11:
+			sizeimage[sizeimage_index] = strtoul(value, 0L, 0);
+			sizeimage_index++;
+			fmts |= FmtSizeImage;
+			break;
 		default:
 			return 0;
 		}
@@ -876,8 +885,8 @@ int parse_selection_target(const char *s, unsigned int &target)
 
 static void print_event(const struct v4l2_event *ev)
 {
-	printf("%ld.%06ld: event %u, pending %u: ",
-			ev->timestamp.tv_sec, ev->timestamp.tv_nsec / 1000,
+	printf("%lld.%06ld: event %u, pending %u: ",
+			static_cast<__u64>(ev->timestamp.tv_sec), ev->timestamp.tv_nsec / 1000,
 			ev->sequence, ev->pending);
 	switch (ev->type) {
 	case V4L2_EVENT_VSYNC:
@@ -922,7 +931,7 @@ static __u32 parse_event(const char *e, const char **name)
 		if (event == V4L2_EVENT_CTRL) {
 			fprintf(stderr, "Missing control name for ctrl event, use ctrl=<name>\n");
 			misc_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	} else if (!strcmp(e, "eos")) {
 		event = V4L2_EVENT_EOS;
@@ -945,16 +954,15 @@ static __u32 parse_event(const char *e, const char **name)
 	if (event == 0) {
 		fprintf(stderr, "Unknown event\n");
 		misc_usage();
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	return event;
 }
 
 bool valid_pixel_format(int fd, __u32 pixelformat, bool output, bool mplane)
 {
-	struct v4l2_fmtdesc fmt;
+	struct v4l2_fmtdesc fmt = {};
 
-	fmt.index = 0;
 	if (output)
 		fmt.type = mplane ? V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_OUTPUT;
@@ -972,7 +980,7 @@ bool valid_pixel_format(int fd, __u32 pixelformat, bool output, bool mplane)
 
 __u32 find_pixel_format(int fd, unsigned index, bool output, bool mplane)
 {
-	struct v4l2_fmtdesc fmt;
+	struct v4l2_fmtdesc fmt = {};
 
 	fmt.index = index;
 	if (output)
@@ -1041,11 +1049,11 @@ static const char *make_devname(const char *device, const char *devname,
 	}
 
 	media_v2_entity *ents = new media_v2_entity[topology.num_entities];
-	topology.ptr_entities = (__u64)ents;
+	topology.ptr_entities = (uintptr_t)ents;
 	media_v2_link *links = new media_v2_link[topology.num_links];
-	topology.ptr_links = (__u64)links;
+	topology.ptr_links = (uintptr_t)links;
 	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
-	topology.ptr_interfaces = (__u64)ifaces;
+	topology.ptr_interfaces = (uintptr_t)ifaces;
 
 	unsigned i, ent_id, iface_id = 0;
 
@@ -1081,7 +1089,7 @@ static const char *make_devname(const char *device, const char *devname,
 	sprintf(newdev, "/dev/char/%d:%d",
 		ifaces[i].devnode.major, ifaces[i].devnode.minor);
 	device = newdev;
-	
+
 err:
 	delete [] ents;
 	delete [] links;
@@ -1136,7 +1144,7 @@ int main(int argc, char **argv)
 			short_options[idx++] = ':';
 		}
 	}
-	while (1) {
+	while (true) {
 		int option_index = 0;
 
 		short_options[idx] = 0;
@@ -1145,7 +1153,7 @@ int main(int argc, char **argv)
 		if (ch == -1)
 			break;
 
-		options[(int)ch] = 1;
+		options[ch] = 1;
 		if (!option_index) {
 			for (i = 0; long_options[i].val; i++) {
 				if (long_options[i].val == ch) {
@@ -1279,7 +1287,7 @@ int main(int argc, char **argv)
 	if (type == MEDIA_TYPE_CANT_STAT) {
 		fprintf(stderr, "Cannot open device %s, exiting.\n",
 			device);
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	switch (type) {
@@ -1299,7 +1307,7 @@ int main(int argc, char **argv)
 	if (type == MEDIA_TYPE_UNKNOWN) {
 		fprintf(stderr, "Unable to detect what device %s is, exiting.\n",
 			device);
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	is_subdev = type == MEDIA_TYPE_SUBDEV;
 	if (is_subdev)
@@ -1316,14 +1324,14 @@ int main(int argc, char **argv)
 	if (fd < 0) {
 		fprintf(stderr, "Failed to open %s: %s\n", device,
 			strerror(errno));
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	verbose = options[OptVerbose];
 	c_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
 
 	if (!is_subdev && doioctl(fd, VIDIOC_QUERYCAP, &vcap)) {
 		fprintf(stderr, "%s: not a v4l2 node\n", device);
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	capabilities = vcap.capabilities;
 	if (capabilities & V4L2_CAP_DEVICE_CAPS)
@@ -1347,12 +1355,12 @@ int main(int argc, char **argv)
 		if (out_fd < 0) {
 			fprintf(stderr, "Failed to open %s: %s\n", out_device,
 					strerror(errno));
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		c_out_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
 		if (doioctl(out_fd, VIDIOC_QUERYCAP, &vcap)) {
 			fprintf(stderr, "%s: not a v4l2 node\n", out_device);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		out_capabilities = vcap.capabilities;
 		if (out_capabilities & V4L2_CAP_DEVICE_CAPS)
@@ -1366,12 +1374,12 @@ int main(int argc, char **argv)
 		if (exp_fd < 0) {
 			fprintf(stderr, "Failed to open %s: %s\n", export_device,
 					strerror(errno));
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		c_exp_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
 		if (doioctl(exp_fd, VIDIOC_QUERYCAP, &vcap)) {
 			fprintf(stderr, "%s: not a v4l2 node\n", export_device);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 
@@ -1380,17 +1388,17 @@ int main(int argc, char **argv)
 	if (wait_for_event == V4L2_EVENT_CTRL && wait_event_id)
 		if (!common_find_ctrl_id(wait_event_id)) {
 			fprintf(stderr, "unknown control '%s'\n", wait_event_id);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	if (poll_for_event == V4L2_EVENT_CTRL && poll_event_id)
 		if (!common_find_ctrl_id(poll_event_id)) {
 			fprintf(stderr, "unknown control '%s'\n", poll_event_id);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	if (epoll_for_event == V4L2_EVENT_CTRL && epoll_event_id)
 		if (!common_find_ctrl_id(epoll_event_id)) {
 			fprintf(stderr, "unknown control '%s'\n", epoll_event_id);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 
 	if (options[OptAll]) {
@@ -1522,7 +1530,7 @@ int main(int argc, char **argv)
 			__u32 seq = 0;
 
 			fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_NONBLOCK);
-			while (1) {
+			while (true) {
 				int res;
 
 				FD_ZERO(&fds);
@@ -1563,7 +1571,7 @@ int main(int argc, char **argv)
 
 			fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_NONBLOCK);
 			epoll_ctl(epollfd, EPOLL_CTL_ADD, fd, &epoll_ev);
-			while (1) {
+			while (true) {
 				int res;
 
 				res = epoll_wait(epollfd, &epoll_ev, 1, -1);
@@ -1601,5 +1609,5 @@ int main(int argc, char **argv)
 	// --all sets --silent to avoid ioctl errors to be shown when an ioctl
 	// is not implemented by the driver. Which is fine, but we shouldn't
 	// return an application error in that specific case.
-	exit(options[OptAll] ? 0 : app_result);
+	std::exit(options[OptAll] ? 0 : app_result);
 }
